import POVME.packages.pymolecule.pymolecule as pymolecule
import numpy
import random
import os
import shutil

# This program demonstrates all the features of pymolecule 2.0, a python framework
# for loading, saving, and manipulating 3D molecular data.

########## First, set up the example directory ##########
if os.path.exists('./example_output'): shutil.rmtree('./example_output')
os.mkdir('./example_output')

########## Saving and Loading Molecular Data ##########

# First, create a molecule object.
molecule = pymolecule.Molecule()
#molecule.fileio.load_pym_into('./example_output/pym_example.pym') # remove this later

# Now, load a PDB file into that object. Calculate the bonds by distance, and
# reindex the serial and resseq fields. Definitions used::
#   pymolecule.Molecule.fileio.load_pdb_into
#   pymolecule.Molecule.fileio.load_pdb_into_using_file_object
#   pymolecule.Molecule.fileio.serial_reindex
#   pymolecule.Molecule.fileio.resseq_reindex
#   pymolecule.Molecule.atoms_and_bonds.create_bonds_by_distance
#   pymolecule.Molecule.selections.select_all_atoms_bound_to_selection
molecule.fileio.load_pdb_into('./example.pdb', True, True, True)

# Save the molecule object as a PDB file. The serial and resseq fields have
# already been reindexed, so we'll skip that. Definitions used:
#   pymolecule.Molecule.fileio.save_pdb
molecule.fileio.save_pdb('./example_output/first_save.pdb', False, False, False)
# You may wish to verify that ./example_output/first_save.pdb has CONECT data and
# sequential serial/resseq fields

# You can also "save" to a string instead of to a file. Definitions used:
#   pymolecule.Molecule.fileio.save_pdb
print molecule.fileio.save_pdb('FILENAME DOES NOT MATTER', False, False, True)[:997]

# Pymolecule can also save the data in the pym format, which is generally
# faster than the PDB format. Definitions used::
#   pymolecule.Molecule.fileio.save_pym
molecule.fileio.save_pym('./example_output/pym_example.pym')

# You can also load from the pym format. Let's create a new molecule
# object and load the pym file into it. Definitions used::
#   pymolecule.Molecule.fileio.load_pym_into
mol_from_pym = pymolecule.Molecule()
mol_from_pym.fileio.load_pym_into('./example_output/pym_example.pym')

########## Atom Selections ##########

# Pymolecule has some excelent selection functions. This example shows
# how to select all the nitrogen and oxygen atoms in PRO residues.
# Definitions used::
#   pymolecule.Molecule.selections.select_atoms
PRO_C_N_selection = molecule.selections.select_atoms({'element_stripped':['C','N'], 'resname_stripped':'PRO'})

# You can also invert that selection. Definitions used:
#   pymolecule.Molecule.selections.invert_selection
not_PRO_C_N_selection = molecule.selections.invert_selection(PRO_C_N_selection)

# You can also get the selection of all the atoms near an existing
# selection (using a user-defined distance cutoff). For example,
# let's get all the atoms within 3 A of the PRO_C_N_selection above.
# Definitions used:
#   pymolecule.Molecule.selections.select_atoms_near_other_selection
near_PRO_C_N_selection = molecule.selections.select_atoms_near_other_selection(PRO_C_N_selection, 3.0)

# Note that near_PRO_C_N_selection doesn't include the atoms specified by
# PRO_C_N_selection. If you want them all together, you can use numpy's
# set-theory routines.
near_PRO_C_N_selection_including_PRO_C_N_selection = numpy.union1d(near_PRO_C_N_selection, PRO_C_N_selection)

# You can also get a selection that includes all the atoms that are in
# the same residues as the atoms of another selection. Definitions used:
#   pymolecule.Molecule.selections.select_atoms_in_same_residue
same_whole_residue_as_near_PRO_C_N_selection_including_PRO_C_N_selection = molecule.selections.select_atoms_in_same_residue(near_PRO_C_N_selection_including_PRO_C_N_selection)

# It's also possible to select all atoms within a bounding box. Definitions
# used:
#   pymolecule.Molecule.selections.select_atoms_in_bounding_box
bounding_box = numpy.array([[-5.0, 60.0, 85.0], [15.0, 80.0, 105.0]])
bounding_box_selection = molecule.selections.select_atoms_in_bounding_box(bounding_box)

# If your pymolecule.Molecule object actually contains multiple molecules,
# you can select all the atoms that belong to the same molecule as a
# user-specified atom. For example, let's select all the atoms that belong
# to the same molecule as the first atom in our pymolecule.Molecule object.
# (This should just be chain A). Note that this only works if you've
# specified the bonds. Definitions used:
#   pymolecule.Molecule.selections.select_atoms_from_same_molecule
same_molecule_as_first_atom_selection = molecule.selections.select_atoms_from_same_molecule([0])

# You can do the same with entire selections, instead of individual indecies.
# For example, let's select all the molecules containing either the first
# or the last atom. This should effectively select both chain A and B.
# Definitions used:
#   pymolecule.Molecule.selections.select_atoms_from_same_molecule
same_molecule_as_first_or_last_atom_selection = molecule.selections.select_atoms_from_same_molecule(numpy.array([0,len(molecule.information.get_coordinates())-1]))

# You can also get the selections of all the unique molecules. This can be
# used to divide a pymolecule.Molecule object containing multiple molecules
# (e.g., chains) into separate molecules. In this case, it will divide
# molecule into chain A and B, which are not connected by any bond and so
# are separate molecules. Note that, again, bonds must have been defined.
# Definitions used:
#   pymolecule.Molecule.selections.selections_of_constituent_molecules
unique_molecule_selections = molecule.selections.selections_of_constituent_molecules()

# You can get a seleciton of all atoms that are bonded to a given atom
# selection. For example, let's get a selection for all the atoms bound to
# the first and 26th atom (not including those atoms themselves). Definitions
# used:
#   pymolecule.Molecule.select_all_atoms_bound_to_selection
bonded_atoms_selection = molecule.selections.select_all_atoms_bound_to_selection(numpy.array([0,25]))

# You can select the atoms of a branch. You need only specify the root atom
# of the branch and the first atom of the branch attached to that root,
# to indicate the directionality. For example, the 3186th atom of molecule
# is a TRP alpha carbon, and the 3189th atom is the TRP beta carbon.
# so using the 3186th and 3189th atom as the root and directionality atom,
# respectively, you can select the alpha carbon + TRP sidechain.
# Definitions used:
#   pymolecule.Molecule.select_branch
TRP_side_chain_and_alpha_carbon = molecule.selections.select_branch(3186, 3189)

# Another common task is to select all atoms. Definitions used:
#   pymolecule.Molecule.selections.select_all
all_atoms = molecule.selections.select_all()

# Any selection can be converted into a molecule object. Definitions used:
#   pymolecule.Molecule.selections.get_molecule_from_selection
PRO_C_N_molecule = molecule.selections.get_molecule_from_selection(PRO_C_N_selection)
not_PRO_C_N_molecule = molecule.selections.get_molecule_from_selection(not_PRO_C_N_selection)
all_atoms_molecule = molecule.selections.get_molecule_from_selection(all_atoms)
near_PRO_C_N_molecule = molecule.selections.get_molecule_from_selection(near_PRO_C_N_selection)
near_PRO_C_N_selection_including_PRO_C_N_molecule = molecule.selections.get_molecule_from_selection(near_PRO_C_N_selection_including_PRO_C_N_selection)
same_whole_residue_as_near_PRO_C_N_selection_including_PRO_C_N_molecule = molecule.selections.get_molecule_from_selection(same_whole_residue_as_near_PRO_C_N_selection_including_PRO_C_N_selection)
same_molecule_as_first_atom_molecule = molecule.selections.get_molecule_from_selection(same_molecule_as_first_atom_selection)
same_molecule_as_first_or_last_atom_molecule = molecule.selections.get_molecule_from_selection(same_molecule_as_first_or_last_atom_selection)
bonded_atoms_molecule = molecule.selections.get_molecule_from_selection(bonded_atoms_selection)
TRP_side_chain_and_alpha_carbon_molecule = molecule.selections.get_molecule_from_selection(TRP_side_chain_and_alpha_carbon)
bounding_box_molecule = molecule.selections.get_molecule_from_selection(bounding_box_selection)
unique_molecules = [molecule.selections.get_molecule_from_selection(sel) for sel in unique_molecule_selections]

# Like any molecule object, these can also be saved to files. Note that here we
# are not reindexing the serial or resseq fields.
PRO_C_N_molecule.fileio.save_pdb('./example_output/selection_PRO_C_N.pdb', False, False)
not_PRO_C_N_molecule.fileio.save_pdb('./example_output/selection_not_PRO_C_N.pdb', False, False)
all_atoms_molecule.fileio.save_pdb('./example_output/selection_all_atoms.pdb', False, False)
near_PRO_C_N_molecule.fileio.save_pdb('./example_output/selection_near_PRO_C_N.pdb', False, False)
near_PRO_C_N_selection_including_PRO_C_N_molecule.fileio.save_pdb('./example_output/selection_near_PRO_C_N_selection_including_PRO_C_N.pdb', False, False)
same_whole_residue_as_near_PRO_C_N_selection_including_PRO_C_N_molecule.fileio.save_pdb('./example_output/selection_same_whole_residue_as_near_PRO_C_N_selection_including_PRO_C_N.pdb', False, False)
same_molecule_as_first_atom_molecule.fileio.save_pdb('./example_output/selection_same_molecule_as_first_atom.pdb', False, False)
same_molecule_as_first_or_last_atom_molecule.fileio.save_pdb('./example_output/selection_same_molecule_as_first_or_last_atom.pdb', False, False)
bonded_atoms_molecule.fileio.save_pdb('./example_output/selection_bonded_atoms.pdb', False, False)
TRP_side_chain_and_alpha_carbon_molecule.fileio.save_pdb('./example_output/selection_TRP_side_chain_and_alpha_carbon.pdb', False, False)
bounding_box_molecule.fileio.save_pdb('./example_output/bounding_box.pdb', False, False)
for index, mol in enumerate(unique_molecules): mol.fileio.save_pdb('./example_output/unique_mols_' + str(index + 1) + '.pdb', False, False)

# There's also a shortcut for making a copy of an existing molecule object,
# so you don't have to create a selection and then a molecule from that selection.
# Definitions used::
#   pymolecule.Molecule.copy
mol_copy = PRO_C_N_molecule.copy()

# Again, like any molecule object, this copy can also be saved.
mol_copy.fileio.save_pdb('./example_output/selection_PRO_C_N_copy.pdb')

# Another common task is to separate a molecule by chain or residue. Pymolecule
# provides special functions to get all these selections. First, I'll demonstrate
# how to get all the chain selections. Definitions used:
#   pymolecule.Molecule.selections.get_chain_divisions
chain_selections = molecule.selections.selections_of_chains()

# chain_selections is a dictionary, where the keys are the chainids and the values
# are the corresponding selections. So, to save each chain to a separate file:
molecule_A = molecule.selections.get_molecule_from_selection(chain_selections['A'])
molecule_A.fileio.save_pdb("./example_output/2HU4_chain_A.pdb")
molecule_B = molecule.selections.get_molecule_from_selection(chain_selections['B'])
molecule_B.fileio.save_pdb("./example_output/2HU4_chain_B.pdb")

# Similarly, you can get the selections of all the residues. Definitions used:
#   pymolecule.Molecule.selections.selections_of_residues
residue_selections = molecule.selections.selections_of_residues()

# residue_selections is a dictionary, where they keys have the form
# {resname}-{resseq}-{chainid} (e.g., THR-461-B) and the values are the corresponding
# selections. So, to save the first five residues to separate files:
for index, residueid in enumerate(residue_selections.keys()):
    if index > 4: break
    residue = molecule.selections.get_molecule_from_selection(residue_selections[residueid])
    residue.fileio.save_pdb('./example_output/2HU4_residue_' + str(index+1) + ".pdb")

# You can also select atoms from two different molecules that come within a user-
# specified distance of each other. Let's identify atoms from chain A and B that come
# within 10 A of each other using select_close_atoms_from_different_molecules_atom_by_atom.
# If the third parameter is set to True, this function performs a simple pair-wise
# distance comparison. It's fairly fast, but will not work well when comparing two
# molecules that are very large. Definitions used:
#   pymolecule.Molecule.selections.select_close_atoms_from_different_molecules
molecule_A_close_atoms_selection, molecule_B_close_atoms_selection = molecule_A.selections.select_close_atoms_from_different_molecules(molecule_B, 10.0, True)

molecule_A_close_atoms = molecule_A.selections.get_molecule_from_selection(molecule_A_close_atoms_selection)
molecule_B_close_atoms = molecule_B.selections.get_molecule_from_selection(molecule_B_close_atoms_selection)

molecule_A_close_atoms.fileio.save_pdb("./example_output/chain_A_close_atoms.pdb")
molecule_B_close_atoms.fileio.save_pdb("./example_output/chain_B_close_atoms.pdb")

# If the third parameter is set to False, a more sophisticated method for identifying
# juxtaposed atoms is used. This method encompasses the whole molecule, each chain,
# and each residue in a bounding sphere and then checks the spheres for overlap to
# minimize the number of expensive pair-wise distance comparisons required. The first
# time it's run, it's slower than when a pairwise comparison is prformed. However, the
# location and radii of the bounding spheres are saved, so that it's much faster the
# second time it's run. This function is also ideal when trying to detect steric
# clashes between two very large molecules. Definitions used:
#   pymolecule.Molecule.selections.select_close_atoms_from_different_molecules
molecule_A_close_atoms_selection_v2, molecule_B_close_atoms_selection_v2 = molecule_A.selections.select_close_atoms_from_different_molecules(molecule_B, 10.0, False)

molecule_A_close_atoms_v2 = molecule_A.selections.get_molecule_from_selection(molecule_A_close_atoms_selection_v2)
molecule_B_close_atoms_v2 = molecule_B.selections.get_molecule_from_selection(molecule_B_close_atoms_selection_v2)

molecule_A_close_atoms_v2.fileio.save_pdb("./example_output/chain_A_close_atoms_v2.pdb")
molecule_B_close_atoms_v2.fileio.save_pdb("./example_output/chain_B_close_atoms_v2.pdb")

########## Manipulating Atomic Coordinates ##########

# PyMolecule also has some useful functions for manipulating molecular data (i.e., 
# rotating and translating molecules, etc.). In order to undo any of these
# manipulations, we must first set the undo point. Definitions used:
#   pymolecule.Molecule.manipulation.set_coordinate_undo_point
molecule.manipulation.set_coordinate_undo_point()

# Now, let's translate the molecule (without rotation) so that the 5th atom is
# at (1.0, 1.0, 1.0) and save it. Definition used:
#   pymolecule.Molecule.manipulation.set_atom_location
molecule.manipulation.set_atom_location(4, numpy.array([1.0, 1.0, 1.0]))
molecule.fileio.save_pdb('./example_output/2HU4_5th_atom_to_pt.pdb')

# We can also move the molecule 2.0 A up the x axis. Definitions used:
#   pymolecule.Molecule.manipulation.translate_molecule
molecule.manipulation.translate_molecule(numpy.array([2.0, 0.0, 0.0]))
molecule.fileio.save_pdb('./example_output/2HU4_translate_2_down_x.pdb')

# There are also functions for rotating functions. For example, here's how
# to rotate a molecule 45 degress around a line connecting the points (1,1,1)
# and (10,0,10). Definitions used:
#   pymolecule.Molecule.manipulation.rotate_molecule_around_a_line_between_points
molecule.manipulation.rotate_molecule_around_a_line_between_points(numpy.array([[1.0, 1.0, 1.0]]), numpy.array([[10.0, 10.0, 10.0]]), numpy.pi*0.25)
molecule.fileio.save_pdb('./example_output/2HU4_rotation_test_1.pdb')

# You can also rotate the molecule 45 degrees around the line formed by
# connecting the 4th and 5th atom. Definitions used:
#   pymolecule.Molecule.manipulation.rotate_molecule_around_a_line_between_atoms
molecule.manipulation.rotate_molecule_around_a_line_between_atoms(3, 4, numpy.pi*0.25)
molecule.fileio.save_pdb('./example_output/2HU4_rotation_test_2_about_4th_5th_atom.pdb')

# The molecule can also be pivoted around a point, in this case (10.0, 10.0, 10.0)
# Definitions used:
#   pymolecule.Molecule.manipulation.rotate_molecule_around_pivot_point
molecule.manipulation.rotate_molecule_around_pivot_point(numpy.array([10.0,10.0,10.0]), numpy.pi*0.25, numpy.pi*0.25, numpy.pi*0.25)
molecule.fileio.save_pdb('./example_output/2HU4_rotation_test_about_pivot_pt.pdb')

# Or the molecule can be pivoted around the coordinates of a given atom, in this case
# the 5th atom. Definitions used:
#   pymolecule.Molecule.manipulation.rotate_molecule_around_pivot_atom
molecule.manipulation.rotate_molecule_around_pivot_atom(4, numpy.pi*0.25, numpy.pi*0.25, numpy.pi*0.25)
molecule.fileio.save_pdb('./example_output/2HU4_rotation_test_about_pivot_5th_atom.pdb')

########## Obtaining Molecular Information ##########

# There are also PyMolecule functions for getting information about the molecule
# like the center of mass, the geometric center, the total mass, etc.
# Definitions used:
#   pymolecule.Molecule.information.get_center_of_mass
#   pymolecule.Molecule.information.get_geometric_center
#   pymolecule.Molecule.information.get_total_mass
#   pymolecule.Molecule.information.get_total_number_of_atoms
#   pymolecule.Molecule.information.get_total_number_of_heavy_atoms
#   pymolecule.Molecule.information.get_bounding_box
print "Center of mass:", molecule.information.get_center_of_mass()
print "Geometric center:", molecule.information.get_geometric_center()
print "Total mass:", molecule.information.get_total_mass()
print "Total number of atoms:", molecule.information.get_total_number_of_atoms()
print "Total number of heavy atoms:", molecule.information.get_total_number_of_heavy_atoms()
print "Bounding box:", molecule.information.get_bounding_box()

# You can also get some information about what atoms are bonded to selected atoms.
# For example, get the number of hydrogen atoms bonded to the 1st atom:
# Definitions used:
#   pymolecule.Molecule.atoms_and_bonds.get_number_of_bond_partners_of_element
print "Number of hydrogen atoms bound to first atom:", molecule.atoms_and_bonds.get_number_of_bond_partners_of_element(0, "H")

# You can also get the index of the first atom with a given element bound to
# a specified atom. Definitions used:
#   pymolecule.Molecule.atoms_and_bonds.get_index_of_first_bond_partner_of_element
print "Index of the first hydrogen atom bound to first atom:", molecule.atoms_and_bonds.get_index_of_first_bond_partner_of_element(0, "H")

# Functions also exist for determining whether a given atom belongs to a protein,
# DNA, or RNA molecule. Consider the 3943rd atom, which is known to belong to a
# protein. Definitions used:
#   pymolecule.Molecule.information.belongs_to_protein
#   pymolecule.Molecule.information.belongs_to_dna
#   pymolecule.Molecule.information.belongs_to_rna
print "3943rd atom belongs to protein?", molecule.information.belongs_to_protein(3942)
print "3943rd atom belongs to DNA?", molecule.information.belongs_to_dna(3942)
print "3943rd atom belongs to RNA?", molecule.information.belongs_to_rna(3942)

########## Constructing Molecules Programatically ##########

# It's also possible to construct molecules programatically, rather than
# loading them from files. For example, here's how to create a new
# molecule object and add five carbon atoms. Definitions used:
#   pymolecule.Molecule.atoms_and_bonds.add_atom
new_mol = pymolecule.Molecule()
new_mol.atoms_and_bonds.add_atom(serial=1, element="C", coordinates = numpy.array([0,0,0]))
new_mol.atoms_and_bonds.add_atom(serial=2, element="C", coordinates = numpy.array([5,5,0]))
new_mol.atoms_and_bonds.add_atom(serial=3, element="C", coordinates = numpy.array([10,0,0]))
new_mol.atoms_and_bonds.add_atom(serial=4, element="C", coordinates = numpy.array([10,10,0]))
new_mol.atoms_and_bonds.add_atom(serial=5, element="C", coordinates = numpy.array([0,10,0]))

# You can also add bonds programatically. Definitions used:
#   pymolecule.Molecule.atoms_and_bonds.add_bond
new_mol.atoms_and_bonds.add_bond(0,2)
new_mol.atoms_and_bonds.add_bond(2,3)
new_mol.atoms_and_bonds.add_bond(3,4)
new_mol.atoms_and_bonds.add_bond(4,0)

# Let's save the file so you can examine it, taking note that the bonds
# are present in the PDB CONECT entries.
new_mol.fileio.save_pdb('./example_output/from_scratch_v1.pdb',False,False)

# There are also functions for deleting atoms and bonds. Definitions used:
#   pymolecule.Molecule.atoms_and_bonds.delete_atom
#   pymolecule.Molecule.atoms_and_bonds.delete_bond
new_mol.atoms_and_bonds.delete_atom(1)
new_mol.atoms_and_bonds.delete_bond(2,1)

# Let's save that PDB as well so you can compare them:
new_mol.fileio.save_pdb('./example_output/from_scratch_v2.pdb',False,False)

########## Comparing Multiple Molecules ##########

# PyMolecule also provides useful functions for comparing multiple molecule objects.
# Let's see if there are any steric clashes between chain A and B, using a large
# 10 A cutoff. As above, if the third parameter is True, a simple pair-wise distance
# comparison is performed. Definitions used:
#   pymolecule.Molecule.other_molecule.steric_clash_with_another_molecule
print "Clash according to steric_clash_with_another_molecule_atom_by_atom:", molecule_A.other_molecule.steric_clash_with_another_molecule(molecule_B, 10.0, True)

# If the third parameter is False (as above), the more sophisticated method for
# identifying juxtaposed atoms is used. Definitions used:
#   pymolecule.Molecule.other_molecule.steric_clash_with_another_molecule
print "Clash according to steric_clash_with_another_molecule:", molecule_A.other_molecule.steric_clash_with_another_molecule(molecule_B, 10.0, False)

# PyMolecule can also be used to identify the minimum distance between two molecules.
# The second parameter is like the third above, determining whether pair-wise or more
# sophisticated methods are used to identify distances. Definitions used:
#   pymolecule.Molecule.other_molecule.get_distance_to_another_molecule
print 'Closest inter-chain distance according to pairwise comparison:', molecule_A.other_molecule.get_distance_to_another_molecule(molecule_B, True)
print 'Closest inter-chain distance according to sophisticated method:', molecule_A.other_molecule.get_distance_to_another_molecule(molecule_B, False)
#### I THINK THE ABOVE, THE BETTER METHOD MIGHT TAKE LONGER. WHY??? CHECK GENERALLY.

# PyMolecule can calculate RMSD values. To demonstrate this, let's look at the first
# 100 atoms of the two chains.
molecule_A_first_hundred = molecule_A.selections.get_molecule_from_selection(numpy.arange(0,100,1,dtype=int))
molecule_B_first_hundred = molecule_B.selections.get_molecule_from_selection(numpy.arange(0,100,1,dtype=int))

# The atoms of these two molecules are in the same order, so we can determine which
# atoms are equivalent based on order alone. Definitions used:
#   pymolecule.Molecule.other_molecule.get_rmsd_order_dependent
print "RMSD between two chains, based on order: ", molecule_A_first_hundred.other_molecule.get_rmsd_order_dependent(molecule_B_first_hundred)

# You can also specify which atoms are equivalent. This is accomplished using a tuple
# of two numpy.array, where each array is comprised of the indecies of self and
# other_mol, respectively, such that equivalent atoms are listed in order. So, for
# example, if (atom1/mol1 = atom6/mol2) and (atom2/mol1 = atom3/mol2) than the tethers
# would be ([1,2], [6,3]). Let's just generate these tethers randomly to demonstrate
# how this functionality is used. Definitions used:
#   pymolecule.Molecule.other_molecule.get_rmsd_equivalent_atoms_specified
mol1_index_in_order = numpy.arange(0,len(molecule_A_first_hundred.information.get_coordinates()),1,dtype=int)
mol2_index_in_order = numpy.arange(0,len(molecule_B_first_hundred.information.get_coordinates()),1,dtype=int)
numpy.random.shuffle(mol1_index_in_order)
numpy.random.shuffle(mol2_index_in_order)

print "RMSD between two chains, equiv atoms specified:", molecule_A_first_hundred.other_molecule.get_rmsd_equivalent_atoms_specified(molecule_B_first_hundred, (mol1_index_in_order, mol2_index_in_order))

##### YOU NEED ORDER INDEPENDENT ONE TOO!!!! WITH FINGERPRINTS???? ######

# PyMolecule also has a function to merge two molecules. Let's recreate the original
# molecule object, with chains A and B in the same object. Note the importance in this
# case of reindexing the serial field when you save, or the bonds may be incorrect.
# Definitions used:
#   pymolecule.Molecule.other_molecule.merge_with_another_molecule
merged = molecule_A.other_molecule.merge_with_another_molecule(molecule_B)
merged.fileio.save_pdb('./example_output/merged.pdb',True,False)

# You can also align portions of molecules to minimize the RMSD distance between them.
# For example, let's align the two chains by the first TRP residue in chain A and the
# last in chain B. Definitions used:
#   pymolecule.Molecule.other_molecule.get_other_molecule_aligned_to_this

molecule_A_first_TRP_selection = molecule_A.selections.select_atoms_in_same_residue(molecule_A.selections.select_atoms({'resname_stripped':'TRP'})[0])
molecule_B_first_TRP_selection = molecule_B.selections.select_atoms_in_same_residue(molecule_B.selections.select_atoms({'resname_stripped':'TRP'})[-1])
tethers = numpy.vstack((molecule_A_first_TRP_selection, molecule_B_first_TRP_selection))
molecule_B_aligned_to_A = molecule_A.other_molecule.get_other_molecule_aligned_to_this(molecule_B, tethers)
molecule_B_aligned_to_A.fileio.save_pdb('./example_output/last_TRP_of_chain_B_aligned_to_first_TRP_of_chain_A.pdb', False, False, False)

########## Random Geometry Functions ##########

# Most geometry functions should be done through numpy. However, there are a
# few functions that have dedicated functions, since they aren't necessarily
# trivial to do in numpy, or because of historical reasons (i.e., they were
# included in older versions of pymolecule)

# For example, you can get the angle between three points. Definitions used:
#   pymolecule.Molecule.geometry.get_angle_between_three_points
print "Angle in radians between 1st, 2nd, and 3rd atoms:", molecule.geometry.get_angle_between_three_points(molecule.information.get_coordinates()[0], molecule.information.get_coordinates()[1], molecule.information.get_coordinates()[2])

# You can also get the dihedral angle between four points. Definitions used:
#   pymolecule.Molecule.geometry.get_dihedral_angle
print "Dihedral angle in radians between 1st, 2nd, 3rd, and 4th atoms:", molecule.geometry.get_dihedral_angle(molecule.information.get_coordinates()[0], molecule.information.get_coordinates()[1], molecule.information.get_coordinates()[2], molecule.information.get_coordinates()[3])

# There's also a function to determine whether or not four points are in
# a plane (within a given tolerance). For example, the 202nd, 203rd, 207th,
# and 208th atoms in molecule belong to the same TRP sidechain and so are
# roughly planar. Definitions used:
#   pymolecule.Molecule.geometry.is_planar
#   pymolecule.Molecule.geometry.get_planarity_deviation
#four_atoms_from_TRP_sidechain_selection = numpy.array([201, 202, 206, 207])
print "Four atoms in the same TRP sidechain are roughly planar?", molecule.geometry.is_planar(molecule.information.get_coordinates()[201], molecule.information.get_coordinates()[202], molecule.information.get_coordinates()[206], molecule.information.get_coordinates()[207], 0.2)



# Calculates the binding site overlap between different conformations of the same protein.
# Started July 9th, 2014
# Celia Wong

import numpy
import sys
import re
import os
import csv

class Trajectory():

    def __init__(self):
        self.coordinates = []
        self.frames = 0
        self.frameToFileName = {}

    # Save each frame in the trajectory as a set
    ''' def read_traj(self,traj_file):

        trajectory = open(traj_file,'r')
        frame_coordinates = []
        counter = []
        count = 0
    '''
    ''' Check first line to see if it contains volume count, if not, incorrect pdb file '''

    ''' for line in trajectory:
            if line[0] == 'E':
                self.coordinates.append(set(frame_coordinates))
                #self.coordinates.append(frame_coordinates)
                #numpy.append(self.coordinates,set(frame_coordinates))
                frame_coordinates = []

                #counter.append(count)
                #count = 0

                self.frames += 1

            elif len(line) > 6 and line[7] == 'V':
                frame_vol = float(re.sub("[^0123456789\.]","",line))
                self.volume.append(frame_vol)

            elif line[0] == 'A':
                # If there is an X present there is no viable coordinates for that line
                if line[17] != 'X':
                    #if (float(line[29:37].strip()),float(line[38:45].strip()),float(line[46:54].strip())) in frame_coordinates:
                    #   print (float(line[29:37].strip()),float(line[38:45].strip()),float(line[46:54].strip()))
                    frame_coordinates.append((float(line[29:37].strip()),float(line[38:45].strip()),float(line[46:54].strip())))
                    #count += 1

        #print counter
        print self.coordinates
        trajectory.close()
    '''


    def read_traj(self,traj_file):
        
        i = 0
        for filename in traj_file:
            self.coordinates.append(set([tuple(dummy_atom) for dummy_atom in numpy.load(filename)]))
            self.frames+= 1
            self.frameToFileName[i] = filename
            i += 1
        #print self.coordinates
        #print self.frameToFileName
        #print "\n"
        return

class Overlap():

    def __init__(self,coordinates):
        self.coordinates = coordinates
        self.volumes = []
        
        for i in range(len(self.coordinates)):
            self.volumes.append(len(self.coordinates[i]))

    def sum_volumes(self,frame1,frame2):
        total = self.volumes[frame1] + self.volumes[frame2]
        return total

    def number_overlap(self,frame1,frame2):
        # must be a set data structure in order to use
        
        #print self.coordinates[frame1]
        #setFrame1 = set([tuple(i) for i in self.coordinates[frame1]])
        #setFrame2 = set([tuple(i) for i in self.coordinates[frame2]])
        setFrame1 = self.coordinates[frame1]
        setFrame2 = self.coordinates[frame2]
        
        if not isinstance(setFrame1,set) or not isinstance(setFrame2,set):
            print "Coordinates must be contained in a set object"
            sys.exit(1)

        num_overlap_points = len(setFrame1.intersection(setFrame2))

        '''Test here for error values '''
        if num_overlap_points > min(len(setFrame1),len(setFrame2)):
            print 'invalid overlap value'
        return num_overlap_points

    # need to calculate the volume that is overlapped
    def volume_overlap(self,frame1,frame2):
        num_overlap_points = self.number_overlap(frame1, frame2)

        overlap_ratio = num_overlap_points / float(len(self.coordinates[frame1]))

        vol_overlap = overlap_ratio * self.volumes[frame1]

        return vol_overlap

    ''' Incorrect formula do not use
    def schrodinger_overlap(self,frame1,frame2):
        vlap = self.number_overlap(frame1,frame2)
        vtotal = self.sum_volumes(frame1,frame2)
        print vlap, vtotal
        normalized_overlap = float((2*vlap)/float(vtotal))
        return normalized_overlap
    '''
    
    def tanimoto_overlap (self,frame1,frame2):

        vlap = self.number_overlap(frame1,frame2)
        vtotal = self.sum_volumes(frame1,frame2) - vlap
        tanimoto = float(vlap/float(vtotal))
        return tanimoto

    def tversky_overlap(self,frame1,frame2):
        ''' Tversky index: overlap/[Va(w/o overlap) + overlap] & overlap/[Vb(w/o overlap) + overlap]
        '''
        volume_overlap = self.number_overlap(frame1,frame2)
        total_volume = self.volumes[frame1]
        #print 'volume_overlap = {0}, total volume = {1}'.format(volume_overlap,total_volume)
        tversky = float(volume_overlap/float(total_volume))
        return tversky

class main():

    def __init__(self,argv):

        command_input = {}
        command_input['traj_file'] = []
        command_input['output_name'] = ''

#        for arg in argv:
#            if '.pdb' in arg:
#                command_input['traj_file'] = arg
#                command_input['output_name'] = command_input['traj_file'].split('.')[0]

        for arg in argv:
            if ('.npy' in arg) and ('frame' in arg):
                command_input['traj_file'].append(arg)
        print command_input['traj_file']
        
        if command_input == []:
            print "You need to specify .npy files to be read"
            sys.exit(1)

        file_input = Trajectory()
        file_input.read_traj(command_input['traj_file'])

        overlap_value = Overlap(file_input.coordinates)

        num_frames = len(file_input.coordinates)
        print "The number of frames found was: {0}".format(num_frames)
        
        ''' TEST TEST TEST '''
        for i in range(len(file_input.coordinates)):
            print "The number of atoms in {0} is {1}".format(file_input.frameToFileName[i],len(file_input.coordinates[i]))
        #print overlap_value.volumes

        '''TESTING HERE'''
#        print 'intersection value: {0}'.format(len(set.intersection(overlap_value.coordinates[0],overlap_value.coordinates[0])))
    
#        overlap_matrix = numpy.empty([file_input.frames,file_input.frames],dtype=float)

        ''' 
            Schrodinger calculation
            Find the overlap value for all possible combinations given the number of frames
        for f1 in range(num_frames):
            for f2 in range(f1,num_frames):
                normalized_overlap = overlap_value.schrodinger_overlap(f1, f2)
                overlap_matrix[f1,f2] = normalized_overlap
                overlap_matrix[f2,f1] = normalized_overlap
        print overlap_matrix
        '''
        '''Record the overlap_matrix in a csv file'''
        #print os.getcwd()
        #numpy.savetxt(os.getcwd()+'/POVME_Schrodinger_matrix_'+command_input['output_name']+'.csv', overlap_matrix, delimiter =',')

        print "Starting Tanimoto calculations"

        print "Overlap Matrix for Tanimoto calculation"
        tanimoto_matrix = numpy.empty([file_input.frames,file_input.frames],dtype=float)

        for f1 in range(num_frames):
            for f2 in range(f1,num_frames):
                tanimoto_result = overlap_value.tanimoto_overlap(f1,f2)
                tanimoto_matrix[f1,f2] = tanimoto_result
                tanimoto_matrix[f2,f1] = tanimoto_result

        print tanimoto_matrix
        #numpy.savetxt(os.getcwd()+'/POVME_Tanimoto_matrix_'+command_input['output_name']+'.csv', tanimoto_matrix, delimiter=',')
        numpy.save(os.getcwd()+'/POVME_Tanimoto_matrix_'+command_input['output_name']+'.npy',tanimoto_matrix)

        print "Starting Tversky calculations"
        tversky_matrix = numpy.empty([file_input.frames,file_input.frames],dtype=float)

        for f1 in range(num_frames):
            for f2 in range(num_frames):
                #print 'length of frame 1 = {0}'.format(len(overlap_value.coordinates[f1]))
                tversky_matrix[f1,f2] = overlap_value.tversky_overlap(f1, f2)

        print tversky_matrix
        #numpy.savetxt(os.getcwd()+'/POVME_Tversky_matrix_'+command_input['output_name']+'.csv',tversky_matrix,delimiter=',')
        numpy.save(os.getcwd()+'/POVME_Tversky_matrix_'+command_input['output_name']+'.npy',tversky_matrix)
        
        
        ''' Saving which index refers to which frame file for use in clustering '''
        
        frames_dict = file_input.frameToFileName
        print frames_dict

        with open('indexMapToFrames.csv','wb') as csvfile:
            fieldnames = ['index','frame']
            writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
            
            for i in frames_dict:
                writer.writerow({"index": i, "frame" : frames_dict[i]})
            
if __name__ == "__main__": main(sys.argv)



#!python

########################################
######## CURRENTLY INCOMPLETE ##########
########################################
1/0





import numpy as np
import cPickle
import glob
from sklearn.preprocessing import StandardScaler
import pylab
#import hsp90Helpers
import POVME.packages.binana.peel
import os 
pdb2Lig = {#'1BYQ':'ADP',
           '1UYF_every50ns':'PU1',
           '1UYI_every50ns':'PUZ',
           '1UYL_every50ns':'NOLIG',
           #'2VCI':'2GJ',
           #'2WI7':'2KL',
           #'2XHR':'C0P',
           #'2YEJ':'ZZ3',
           #'2YEJ':'XQK',
           #'3B26':'B2L',
           '3D0B_every50ns':'SNX',
           #'3HEK':'BD0',
           #'3K98':'1RC',
           #'3K99':'PFT',
           #'3RKZ':'06T', 
           #'3RLR':'3RR',
           #'4CWN':'6LV',
           #'4FCR':'0TM',
           #'4LWE':'FJ2',
           '4R3M_every50ns':'JR9',
           #'4W7T':'3JC'
           } 
sortedPdbs = pdb2Lig.keys()
sortedPdbs.sort()



every = 1

analysisPath = '../../2-POVME_analysis/'
#groupPrefixes = glob.glob('%s/????' %(analysisPath))
#groupPrefixes = [i.replace(analysisPath+'/','') for i in groupPrefixes]
#print groupPrefixes

# data = open('indexMapToFrames.csv').readlines()
# datasp = [i.split(',') for i in data]
# indices = [int(i[0]) for i in datasp]
# prefixes = [i[1].split('/')[-1].split('_')[0] for i in datasp]

index2frameData = open('indexMapToFrames.csv').readlines()

prefix2indices = {}
prefix2frames = {}
index2prefix = {}
prefixList = []
frameList = []
for line in index2frameData:
    index = int(line.split(',')[0])
    prefix = line.split('/')[-1].split('_')[0]
    frame = int(line.split('/')[-1].split('_')[-1].replace('.npy',''))
    frameList.append(frame)
    prefix2indices[prefix] = prefix2indices.get(prefix,[]) + [int(index)]
    prefix2frames[prefix] = prefix2frames.get(prefix,[]) + [int(frame)]
    index2prefix[index] = prefix

#sortedPrefixes = prefix2indices.keys()
#sortedPrefixes.sort()


## Load all relevant frames
print "Loading frames"
pointsSetList = []
frameStringList = []
allPointsSet = set()
prefixList = []
for prefix in sortedPdbs:
    maxFrame = max(prefix2frames[prefix])
    framesToLoad = range(1,maxFrame,every)
    for frame in framesToLoad:
        frameString = '%s_frame_%i' %(prefix, frame)
        frameFilename = '%s/%s/%s.npy' %(analysisPath,
                                         prefix,
                                         frameString)
        prefixList.append(prefix)
        frameStringList.append(frameString)
        pointsSet = set([tuple(i) for i in np.load(frameFilename)])
        pointsSetList.append(pointsSet)
        allPointsSet = allPointsSet.union(pointsSet)

        
print "Vectorizing frames"
coord2vectPos = dict([(coord, i) for i, coord in enumerate(allPointsSet)])
vectPos2Coord = dict([(i, coord) for i, coord in enumerate(allPointsSet)])
nVectPos = len(allPointsSet)
nFrames = len(frameStringList)
vecPosMatrix = np.zeros((nFrames, nVectPos), dtype=np.bool)
for f1 in range(nFrames):
    for point in pointsSetList[f1]:
        vecPosMatrix[f1,coord2vectPos[point]] = 1
#print vecPosMatrix
print 'vecPosMatrix.shape is ', vecPosMatrix.shape

print "Making X_std"

normalizeFeatures = 'Mean'
if normalizeFeatures == 'MeanAndStd':
    X_std = StandardScaler().fit_transform(vecPosMatrix)
elif normalizeFeatures == 'Mean':
    X_std = vecPosMatrix - np.mean(vecPosMatrix,0)

from sklearn.decomposition import PCA as sklearnPCA
print "Making sklearn_pca"
nComponents = 100
sklearn_pca = sklearnPCA(n_components=nComponents)
print "Making Y_sklearn"
sklearn_pca_hash = str(hash(tuple(X_std.flatten()[::100])))[-10:]
pca_result_hash = 'y_sklearn_hash_%s.cpickle' %(sklearn_pca_hash)
if not(os.path.exists(pca_result_hash)):
    Y_sklearn = sklearn_pca.fit_transform(X_std)
    with open(pca_result_hash,'wb') as of:
        cPickle.dump(Y_sklearn, of)
else:
    Y_sklearn = cPickle.load(open(pca_result_hash))

print 'Y_sklearn.shape', Y_sklearn.shape
for col in range(Y_sklearn.shape[1]):
    print col, np.mean(abs(Y_sklearn[:,col]))

twoD = True
threeD = False
plotExplainedVariance = True

savePcDxFiles = True
nCircImgPlaces = 20
#showMolAtAvgPos = True
#contourNotScatter = False


showMolAtAvgPos = False
contourNotScatter = True

scatters = []
if contourNotScatter:
    colors = ['b', 'c', 'g', 'm', 'r', 'k', '0.5'] 
    markers = ['solid','dashed','dashdot']

if not(contourNotScatter):
    colors = ['b', 'c', 'g', 'm', 'r', 'k'] 
    markers = ['x', 'o', 's', 'v']


colorsMarkers = [[(j,i) for i in markers] for j in colors] # This will be a list of lists
colorsMarkers = [i for sl in colorsMarkers for i in sl] # so we flatten it


# The ligand figure circular arrangement will have a radius of the PC value range times this factor
radiusRangeFactor = 0.65
from matplotlib.offsetbox import AnnotationBbox, OffsetImage
import scipy.spatial.distance as ssd
import itertools
if twoD:
    combos = list(itertools.combinations(range(4),2))
    for index, (pc_x, pc_y) in enumerate(combos):
        xMin = min(Y_sklearn[:,pc_x])
        xMax = max(Y_sklearn[:,pc_x])
        yMin = min(Y_sklearn[:,pc_y])
        yMax = max(Y_sklearn[:,pc_y])
        #print 'xMin, xMax, yMin, yMax', xMin, xMax, yMin, yMax
        # These ranges define how the circular positions of ligand figures are placed. The algorithm needs a circle for assignment, but will convert it to an ellipse for drawing
        
        yRange = yMax - yMin
        xRange = yRange
        #recoverXRangeFactor = (float(xMax - xMin) / yRange) ** 0.5
        recoverXRangeFactor = 1.
        
        #yRange = max(Y_sklearn[:,pc_y]) - min(Y_sklearn[:,pc_y])
        #xOffset = -np.mean(Y_sklearn[:,pc_x])
        #yOffset = -np.mean(Y_sklearn[:,pc_y])
        xOffset = xMin + (xRange/2)
        yOffset = yMin + (yRange/2)
        xyCircPlaces = [[(xRange*np.sin(i))+xOffset,
                         (yRange*np.cos(i))+yOffset] 
                         for i in np.arange(0,2*np.pi, 2*np.pi/nCircImgPlaces)]
        xyCircPlaces = np.array(xyCircPlaces)
        xyCircPlaces *= radiusRangeFactor
        circPlacesTaken = np.zeros((nCircImgPlaces))
        
        # Prioritize which molecule reps to draw first
        toPlot = []
        for pdb, (color, marker) in zip(sortedPdbs, colorsMarkers):
            indices = [i for i,thisPrefix in enumerate(prefixList) if thisPrefix==pdb]
            indices = np.array(indices)
            xs = Y_sklearn[indices,pc_x]
            ys = Y_sklearn[indices,pc_y]
            #angle = np.arctan2(np.mean(xs),np.mean(ys))
            #priority = angle
            xAvg = np.mean(xs) - xOffset
            yAvg = np.mean(ys) - yOffset
            priority = ((xAvg**2) + (yAvg**2)) ** 0.5
            priority = 1.0/priority
            #priority = np.random.random()
            #distanceToSpots = ssd.cdist([[xAvg,yAvg]],
            #                            xyCircPlaces)
            #priority = np.min(distanceToSpots)
            toPlot.append((pdb,color,marker,priority))
        #print 'Priority values:', [i[3] for i in toPlot]
        toPlot.sort(key=lambda x:x[3])
        
        
        scatters = []
        #print colorsMarkers
        #ax = pylab.subplot(1,1,index+1)
        ax = pylab.subplot(1,1,1)
        #for pdb, (color, marker) in zip(sortedPdbs, colorsMarkers):
        for pdb, color, marker, angle in toPlot:
            indices = [i for i,thisPrefix in enumerate(prefixList) if thisPrefix==pdb]
            indices = np.array(indices)
            xs = Y_sklearn[indices,pc_x]
            ys = Y_sklearn[indices,pc_y]

            if not(contourNotScatter):
                scatter = pylab.scatter(xs, ys,
                                        color=color,
                                        marker=marker)
            if contourNotScatter:
                scatter = pylab.plot([], [],
                                     color=color,
                                     linestyle=marker
                                     )

                ### Contour code ###
                nBins = 100
                contourLevel = 0.03
                xBins = np.arange(xMin, xMax+0.0001, (xMax-xMin)/nBins)
                yBins = np.arange(yMin, yMax+0.0001, (yMax-yMin)/nBins)
                hist2d, xEdges, yEdges = np.histogram2d(xs, ys, bins=[xBins, yBins])
                import scipy.ndimage.filters
                hist2d = scipy.ndimage.filters.gaussian_filter(hist2d,3.5)
                print xBins[:-1]
                print
                print yBins[:-1]
                print
                print hist2d
                #xBins += [xBins[-1]+(xRange/25)]
                #yBins += [yBins[-1]+(yRange/25)]
                contour = pylab.contour(xBins[:-1],
                                        yBins[:-1],
                                        hist2d.T,
                                        [contourLevel],
                                        #[contourLevel,100],
                                        label=pdb,
                                        colors=color,
                                        linestyles=marker,
                                        #hatches = ['/','\\','|','-','+','x','o','O','.','*']*2
                                        )
                pylab.clabel(contour,
                             fmt={contourLevel:pdb},
                             #fontsize='large'
                             )
                ### End countour code ###
            pylab.xlabel('PC %s' %(pc_x+1))
            pylab.ylabel('PC %s' %(pc_y+1))
            if showMolAtAvgPos:
                #print ax
                #size = (150,150)
                #img = hsp90Helpers.getLigandAsImg(pdb, size=size)
                img = hsp90Helpers.getLigandAsImg(pdb)
                #pylab.figimage(np.array(img),np.mean(xs), np.mean(ys))
                imageBox = OffsetImage(np.array(img), zoom=1.0)
                xAvg = np.mean(xs)
                yAvg = np.mean(ys)

                distanceToSpots = ssd.cdist([[xAvg,yAvg]],
                                            xyCircPlaces)
                closestSpots = np.argsort(distanceToSpots)[0]
                spotFound = False
                for spotIndex in closestSpots:
                    if circPlacesTaken[spotIndex]==0:
                        thisSpot = xyCircPlaces[spotIndex,:]
                        circPlacesTaken[spotIndex] = 1
                        spotFound = True
                        break
                if spotFound == False:
                    1/0
                
                ab = AnnotationBbox(imageBox, (xAvg, yAvg),
                                    #xybox=size,
                                    #xybox=((1.5+np.random.random((1)))*xAvg,
                                    #       (1.5+np.random.random((1)))*yAvg),
                                    xybox=(((thisSpot[0]-xOffset)*recoverXRangeFactor)+xOffset,
                                           thisSpot[1]),
                                    #xybox=(thisSpot),
                                    #xycoords='data',
                                    #xycoords=(1.5*xAvg, 1.5*yAvg),
                                    #boxcoords="offset points",
                                    #boxcoords=(1.5*xAvg, 1.5*yAvg),
                                    #pad=0.5,
                                    pad=0.1,
                                    #arrowprops=dict(arrowstyle="->",
                                    arrowprops=dict(arrowstyle="simple",
                                                    facecolor='k'
                                                    #connectionstyle="angle,angleA=0,angleB=90,rad=3"
                                                    )
                                    )
                ax.add_artist(ab)

            scatters.append((scatter,pdb))
        scatters.sort(key=lambda x:x[1])
        if not(contourNotScatter):
            pylab.legend([i[0] for i in scatters], sortedPdbs)
        pylab.show()

if threeD:
    
    import matplotlib.pyplot as plt
    from mpl_toolkits.mplot3d import Axes3D
    fig = plt.figure()
    ax = fig.add_subplot(111, projection='3d')
    for pdb, (color, marker) in zip(sortedPdbs, colorsMarkers):
        indices = [i for i,thisPrefix in enumerate(prefixList) if thisPrefix==pdb]
        indices = np.array(indices)
        xs = Y_sklearn[indices,0]
        ys = Y_sklearn[indices,1]
        zs = Y_sklearn[indices,2]
        scatter = pylab.scatter(xs, ys, zs=zs,
                                color=color,
                                marker=marker)
        scatters.append(scatter)
    pylab.legend(scatters, sortedPdbs)
    pylab.show()

if plotExplainedVariance:
    EVR = sklearn_pca.explained_variance_ratio_
    pylab.bar(np.arange(0.5, 0.5+len(EVR)), EVR)
    pylab.plot([sum(EVR[:i]) for i in range(len(EVR))])
    pylab.show()





templateVisualizePc = '''mol new {%s} waitfor all
#mol addrep !MOLID!
mol modmaterial 0 top HardPlastic
mol modstyle 0 top Isosurface 0.02500000 0 0 0 1 1
#mol modstyle 0 !MOLID! Isosurface 0.02500000 0 2 1 1 1
#green
mol modcolor 0 top ColorID 12

mol addfile {%s} waitfor all
mol addrep top
#mol addrep !MOLID!
mol modmaterial 1 top HardPlastic
mol modstyle 1 top Isosurface -0.02500000 1 0 0 1 1
#mol modstyle 1 !MOLID! Isosurface -0.02500000 1 2 1 1 1
#red
mol modcolor 1 top ColorID 1


'''

    
if savePcDxFiles:
    loadAllScript = 'display rendermode GLSL\ndisplay projection Orthographic\n'
    print 'Saving Principal Component DX files'
    for pc_ind in range(nComponents):
        print "Saving DX file for PC %i" %(pc_ind+1)
        pcPoints = np.zeros((vecPosMatrix.shape[1],4))
        for vectPos in vectPos2Coord:
            coord = vectPos2Coord[vectPos]
            pcPoints[vectPos,:3] = coord
            pcPoints[vectPos,3] = sklearn_pca.components_[pc_ind,vectPos]
        m_fm = peel.featureMap.fromOffGridPts(pcPoints, 1., justCoords=False)
        pcFilename = 'PC%i.dx' %(pc_ind+1)
        m_fm.write_dx_file(pcFilename)
        loadAllScript += templateVisualizePc.replace('!MOLID!',str(pc_ind)) %(pcFilename, pcFilename)
    with open('loadAllPcs.vmd', 'wb') as of:
        of.write(loadAllScript)
    
print "Done!"




from setuptools import setup


setup(
  name = 'POVME',
  packages = ['POVME',
              'POVME.packages',
              'POVME.packages.pymolecule',
              'POVME.packages.binana',
              'POVME.packages.clustering'],
  scripts = ['POVME/packages/clustering/binding_site_overlap.py',
             'POVME/packages/clustering/cluster.py',
             'POVME/POVME3.py',
             'POVME/packages/clustering/pocketPointsPca.py'],
  package_dir={'POVME': 'POVME'},
  package_data={'POVME': ['examples/*/*.*','examples/*/*/*.*'#,'POVME/examples/*/*/*/*'
                          ]},        
  version = '3.0.22',
  description = 'POVME (Pocket VOlume MEasurer) is a Python package for extracting actionable information from ensembles of protein structures for use in drug design.',
  author = 'Jeff Wagner',
  author_email = 'jwagnerjpl@gmail.com',
  url = 'https://github.com/POVME/POVME',
  download_url = 'https://github.com/POVME/POVME/tarball/2.0',
  setup_requires=['numpy','scipy'],
  #build_requires=['scipy','numpy'],
  #install_requires=[''],
  keywords = ['protein folding', 'protein', 'pocket volume measurer', 'computational protein folding', 'protein structures', 'drug design'],
  classifiers = [
    "Development Status :: 4 - Beta",
    "Environment :: Console",
    "Intended Audience :: Science/Research",
    "License :: OSI Approved :: MIT License",
    "Natural Language :: English",
    "Programming Language :: Python :: 2.7",
    "Topic :: Scientific/Engineering :: Bio-Informatics",
    "Topic :: Scientific/Engineering :: Chemistry"
  ]
)



import os
import shutil
import glob


systemNames = [i.split('/')[-1].replace('_aligned.pdb','') for i in glob.glob('../1-trajectories/*_aligned.pdb')]
dashTList = ' '.join(['-t !!!SYSTEM NAME!!!_:../../1-trajectories/!!!SYSTEM NAME!!!_aligned.pdb'.replace('!!!SYSTEM NAME!!!',systemName) for systemName in systemNames])

runAllText = []


systemName = 'ALL'
if os.path.exists(systemName):
    shutil.rmtree(systemName)
os.mkdir(systemName)
ALL_frameList = ['../../2-POVME_analysis/%s/%s_frameInfo/%s_frame_*[0-9].npy' %(i,i,i) for i in systemNames]
runAllScript = '''

binding_site_overlap.py -f !!ALL FRAME LIST!!

cluster.py -n 5 -m tanimoto_matrix.npy -i indexMapToFrames.csv !!DASH T LIST!! 
'''
runAllScript = runAllScript.replace('!!ALL FRAME LIST!!',' '.join(ALL_frameList))
runAllScript = runAllScript.replace('!!DASH T LIST!!', dashTList)
with open('ALL/runClustering_ALL.sh','wb') as of:
    of.write(runAllScript)

with open('ALL/bsubRunScript.sh','wb') as of:
    of.write('''#ln -s ../../../../packages/clustering/binding_site_overlap.py 
#ln -s ../../../../packages/clustering/cluster.py
. ./runClustering_ALL.sh 
''')
    
    



#!python

# Calculates the binding site overlap between sets of POVME outputs.
# Started July 9th, 2014
# Celia Wong
# Advised by Jeff Wagner
# Amaro Lab, UCSD

import numpy
import sys
import re
import os
import csv
from optparse import OptionParser

class Trajectory():

    def __init__(self):
        self.coordinates = []
        self.aromatic_coordinates = []
        self.hbondAcceptor_coordinates = []
        self.hbondDonor_coordinates = []
        self.frames = 0
        self.frameToFileName = {}
        
        self.volumetric_filename = []
        self.aromatic_filename = []
        self.hbondAcceptor_filename = []
        self.hbondDonor_filename = []
        

    #traj_file is a list containing all the filenames of npy files found
    #Must run colored vs uncolored from different directories else uncolored will be reading too many files
     
    #2 pass run, first pass for volumentric only
    #Second pass allow for other wanted files assuming the color flag is set (aromatic, hbondAcceptor, hbondDonor)
    def read_traj(self,traj_file, color):
                
        #check regex for volumetric npy files
        expr = re.compile('frame_\d*.npy')
        #frameNo = re.findall(traj_file, 'frame_([0-9]+).npy')
        #frameNo = re.findall(traj_file, 'frame_([0-9]+)_aromatic.npy')
        sortedFramesAndNames = [(int(re.findall('frame_([0-9]+).npy',name)[0]), name) for name in traj_file]
        sortedFramesAndNames.sort(key=lambda x:x[0])
        sortedNames = [i[1] for i in sortedFramesAndNames]
        print sortedNames
        self.frames = len(sortedNames)
        for index, fileName in enumerate(sortedNames):
            self.volumetric_filename.append(fileName)
            self.coordinates.append(set([tuple(dummy_atom) for dummy_atom in numpy.load(fileName)]))
            self.frameToFileName[index] = fileName
            
        #count = 0
        #for i in range(len(traj_file)):
        #    match_value = expr.search(traj_file[i])
        #    if match_value:
        #        self.volumetric_filename.append(traj_file[i])
        #        self.frames += 1
        #        self.coordinates.append(set([tuple(dummy_atom) for dummy_atom in numpy.load(traj_file[i])]))
        #        self.frameToFileName[count] = traj_file[i]
        #        count += 1
                
                
        #print self.volumetric_filename
        
        #Regex to find aromatic, hbondAcceptor, and hbondDonor files
        #This should guarantee that the frames are in the same order for volumetric and all color metrics
        
        if (color):
            for i in range(len(self.volumetric_filename)):
                aromatic_file = self.volumetric_filename[i].strip('.npy')+'_aromatic.npy'
                hbondAcc_file = self.volumetric_filename[i].strip('.npy')+'_hbondAcceptor.npy'
                hbondDon_file = self.volumetric_filename[i].strip('.npy')+'_hbondDonor.npy'
                self.aromatic_filename.append(aromatic_file)
                self.aromatic_coordinates.append(set([tuple(dummy_atom[:3]) for dummy_atom in numpy.load(aromatic_file) if dummy_atom[3] > 0.02]))
                self.hbondAcceptor_filename.append(hbondAcc_file)
                self.hbondAcceptor_coordinates.append(set([tuple(dummy_atom[:3]) for dummy_atom in numpy.load(hbondAcc_file) if dummy_atom[3] > 0.02]))
                self.hbondDonor_filename.append(hbondDon_file)
                self.hbondDonor_coordinates.append(set([tuple(dummy_atom[:3]) for dummy_atom in numpy.load(hbondDon_file) if dummy_atom[3] > 0.02]))
        

class Overlap():

    def __init__(self,coordinates):
        self.coordinates = coordinates
        self.volumes = []
        
        for i in range(len(self.coordinates)):
            self.volumes.append(len(self.coordinates[i]))

    def sum_volumes(self,frame1,frame2):
        total = self.volumes[frame1] + self.volumes[frame2]
        return total

    def number_overlap(self,frame1,frame2):
        # must be a set data structure in order to use
        
        setFrame1 = self.coordinates[frame1]
        setFrame2 = self.coordinates[frame2]
        
        if not isinstance(setFrame1,set) or not isinstance(setFrame2,set):
            print "Coordinates must be contained in a set object"
            sys.exit(1)

        num_overlap_points = len(setFrame1.intersection(setFrame2))
        
        '''Test here for error values '''
        if num_overlap_points > min(len(setFrame1),len(setFrame2)):
            print 'invalid overlap value'
        return num_overlap_points

    # need to calculate the volume that is overlapped
    def volume_overlap(self,frame1,frame2):
        num_overlap_points = self.number_overlap(frame1, frame2)

        overlap_ratio = num_overlap_points / float(len(self.coordinates[frame1]))

        vol_overlap = overlap_ratio * self.volumes[frame1]

        return vol_overlap
    
    def tanimoto_overlap (self,vlap,frame1,frame2):
        #vlap = self.number_overlap(frame1,frame2)
        vtotal = self.sum_volumes(frame1,frame2) - vlap
        tanimoto = float(vlap/float(vtotal))
        return tanimoto

    def tversky_overlap(self,vlap,frame1,frame2):
        ''' Tversky index: overlap/[Va(w/o overlap) + overlap] & overlap/[Vb(w/o overlap) + overlap]
        '''
        #volume_overlap = self.number_overlap(frame1,frame2)
        total_volume = self.volumes[frame1]
        #print 'volume_overlap = {0}, total volume = {1}'.format(volume_overlap,total_volume)
        tversky = float(vlap/float(total_volume))
        return tversky

''' Helper function needed to get all the input files from optparse '''
def vararg_callback(option, opt_str, value, parser):
    assert value is None
    value = []
        
    for arg in parser.rargs:
        if ((arg[:1] == "-" or arg[:2] == "--") and len(arg) > 1):
            break
        value.append(arg)
            
    del parser.rargs[:len(value)]
    setattr(parser.values, option.dest, value)

class main():

    def __init__(self,argv):
        
        if len(argv) == 1:
            print "Cannot run binding_site_overlap.py: Need to specify .npy files to be read"
            sys.exit(1)
        parser = OptionParser()
        
        parser.add_option("-f", dest = "filename", action = "callback", callback = vararg_callback, help = "All files from POVME that you want to run similarity calculations on.")
        parser.add_option("-c", action = "store_true", dest="color", help = "run similarity calculations on colored output from POVME")
        parser.add_option("--csv", action = "store_true", dest="csv", help = "save human-readable CSV distance matrices.")
        (options, args) = parser.parse_args(argv)
        
        #print command_input['traj_file']
        
        file_input = Trajectory()
        #file_input.read_traj(argv)
        file_input.read_traj(options.filename, options.color)



        ''' Saving which index refers to which frame file for use in clustering '''
        
        frames_dict = file_input.frameToFileName
        with open('indexMapToFrames.csv','wb') as csvfile:
            fieldnames = ['index','frame']
            writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
            
            for i in frames_dict:
                writer.writerow({"index": i, "frame" : frames_dict[i]})
        
        overlap_value = Overlap(file_input.coordinates)
        aromatic_overlap = Overlap(file_input.aromatic_coordinates)
        hbondAcc_overlap = Overlap(file_input.hbondAcceptor_coordinates)
        hbondDon_overlap = Overlap(file_input.hbondDonor_coordinates)

        num_frames = len(file_input.coordinates)
        print "The number of frames found was: {0}".format(num_frames)
        
        '''Make a matrix of number of overlapping coordinates first'''
        
        # Always calculate the overlapping points for volumetric
        overlapStyle = 2
        num_overlap = numpy.empty([file_input.frames,file_input.frames], dtype=float)
        if overlapStyle == 1:
            for f1 in range(num_frames):
                for f2 in range(f1,num_frames):
                    num_overlap[f1,f2] = overlap_value.number_overlap(f1, f2)
        elif overlapStyle == 2:
            allPointsSet = set()
            for f1 in range(num_frames):
                for coord in file_input.coordinates[f1]:
                    allPointsSet.add(coord)
            coord2vectPos = dict([(coord, i) for i, coord in enumerate(allPointsSet)])
            nVectPos = len(allPointsSet)
            vecPosMatrix = numpy.zeros((num_frames, nVectPos), dtype=numpy.bool)
            for f1 in range(num_frames):
                for point in file_input.coordinates[f1]:
                    vecPosMatrix[f1,coord2vectPos[point]] = 1
            for f1 in range(num_frames):
                for f2 in range(f1,num_frames):
                    #raise Exception('There\'s a problem here. In comparing two frames in chris condon\'s pockets, values >1 were found in the tanimoto matrix.')
                    num_overlap[f1,f2] = numpy.count_nonzero(vecPosMatrix[f1,:] & vecPosMatrix[f2,:])
                    num_overlap[f2,f1] = num_overlap[f1,f2]
                    
        
        # Only calculate the colored option if the color option was set
        if (options.color):
            aromatic_num_overlap = numpy.empty([file_input.frames,file_input.frames], dtype=float)
            for f1 in range(num_frames):
                for f2 in range(num_frames):
                    aromatic_num_overlap[f1,f2] = aromatic_overlap.number_overlap(f1, f2)
        
        
            hbondAcc_num_overlap = numpy.empty([file_input.frames,file_input.frames], dtype=float)
            for f1 in range(num_frames):
                for f2 in range(num_frames):
                    hbondAcc_num_overlap[f1,f2] = hbondAcc_overlap.number_overlap(f1, f2)
        
        
            hbondDon_num_overlap = numpy.empty([file_input.frames,file_input.frames], dtype=float)
            for f1 in range(num_frames):
                for f2 in range(num_frames):
                    hbondDon_num_overlap[f1,f2] = hbondDon_overlap.number_overlap(f1, f2)
                    
                    
        '''Record the overlap_matrix in a csv file'''

        print "Starting Tanimoto calculations"

        tanimoto_matrix = numpy.empty([file_input.frames,file_input.frames],dtype=float)
        
        if (options.color):
            aromatic_tanimoto_matrix = numpy.empty([file_input.frames,file_input.frames],dtype=float)
            hbondAcc_tanimoto_matrix = numpy.empty([file_input.frames,file_input.frames],dtype=float)
            hbondDon_tanimoto_matrix = numpy.empty([file_input.frames,file_input.frames],dtype=float)
            colored_tanimoto_matrix = numpy.empty([file_input.frames,file_input.frames],dtype=float)

        for f1 in range(num_frames):
            for f2 in range(f1,num_frames):
                tanimoto_result = overlap_value.tanimoto_overlap(num_overlap[f1,f2],f1,f2)
                tanimoto_matrix[f1,f2] = tanimoto_result
                tanimoto_matrix[f2,f1] = tanimoto_result
                if (options.color):
                    aromatic_result = aromatic_overlap.tanimoto_overlap(aromatic_num_overlap[f1,f2],f1,f2)
                    aromatic_tanimoto_matrix[f1,f2] = aromatic_result
                    aromatic_tanimoto_matrix[f2,f1] = aromatic_result
                    hbondAcc_result = hbondAcc_overlap.tanimoto_overlap(hbondAcc_num_overlap[f1,f2],f1,f2)
                    hbondAcc_tanimoto_matrix[f1,f2] = hbondAcc_result
                    hbondAcc_tanimoto_matrix[f2,f1] = hbondAcc_result
                    hbondDon_result = hbondDon_overlap.tanimoto_overlap(hbondDon_num_overlap[f1,f2],f1,f2)
                    hbondDon_tanimoto_matrix[f1,f2] = hbondDon_result
                    hbondDon_tanimoto_matrix[f2,f1] = hbondDon_result
                
                    average_similarity = (aromatic_result + hbondAcc_result + hbondDon_result + tanimoto_result)/4
                    colored_tanimoto_matrix[f1,f2] = average_similarity             
                    colored_tanimoto_matrix[f2,f1] = average_similarity
                
        print "Overlap Matrix for Tanimoto calculation"
        print tanimoto_matrix
        
        if (options.color):
            print "Aromatic"
            print aromatic_tanimoto_matrix
            print "Hydrogen Bond Acceptor"
            print hbondAcc_tanimoto_matrix
            print "Hydrogen Bond Donor"
            print hbondDon_tanimoto_matrix
            
        print "\n"
        if (options.csv):
            numpy.savetxt(os.getcwd()+'/POVME_Tanimoto_matrix.csv', tanimoto_matrix, delimiter=',')
        
        if (options.color) and (options.csv):
            numpy.savetxt(os.getcwd()+'/POVME_Tanimoto_matrix_aromatic.csv', aromatic_tanimoto_matrix, delimiter=',')
            numpy.savetxt(os.getcwd()+'/POVME_Tanimoto_matrix_hbondAcceptor.csv', hbondAcc_tanimoto_matrix, delimiter=',')
            numpy.savetxt(os.getcwd()+'/POVME_Tanimoto_matrix_hbondDonor.csv', hbondDon_tanimoto_matrix, delimiter=',')
            numpy.savetxt(os.getcwd()+'/POVME_Tanimoto_matrix_colored.csv', colored_tanimoto_matrix, delimiter=',')
            
        numpy.save('tanimoto_matrix.npy', tanimoto_matrix)
        
        if (options.color):
            numpy.save('aromatic_tanimoto_matrix.npy', aromatic_tanimoto_matrix)
            numpy.save('hbondAcc_tanimoto_matrix.npy', hbondAcc_tanimoto_matrix)
            numpy.save('hbondDon_tanimoto_matrix.npy', hbondDon_tanimoto_matrix)


        print "Starting Tversky calculations"
        tversky_matrix = numpy.empty([file_input.frames,file_input.frames],dtype=float)
        if (options.color):
            aromatic_tversky_matrix = numpy.empty([file_input.frames,file_input.frames],dtype=float)
            hbondAcc_tversky_matrix = numpy.empty([file_input.frames,file_input.frames],dtype=float)
            hbondDon_tversky_matrix = numpy.empty([file_input.frames,file_input.frames],dtype=float)
            colored_tversky_matrix = numpy.empty([file_input.frames,file_input.frames],dtype=float)
        

        for f1 in range(num_frames):
            for f2 in range(num_frames):
                #print 'length of frame 1 = {0}'.format(len(overlap_value.coordinates[f1]))
                tversky_matrix[f1,f2] = overlap_value.tversky_overlap(num_overlap[f1,f2],f1, f2)
                if (options.color):
                    aromatic_tversky_matrix[f1,f2] = aromatic_overlap.tversky_overlap(aromatic_num_overlap[f1,f2], f1, f2)
                    hbondAcc_tversky_matrix[f1,f2] = hbondAcc_overlap.tversky_overlap(hbondAcc_num_overlap[f1,f2], f1, f2)
                    hbondDon_tversky_matrix[f1,f2] = hbondDon_overlap.tversky_overlap(hbondDon_num_overlap[f1,f2], f1, f2)
                
                    colored_tversky_matrix[f1,f2] = (hbondAcc_tversky_matrix[f1,f2] + hbondDon_tversky_matrix[f1,f2] + tversky_matrix[f1,f2] + aromatic_tversky_matrix[f1,f2])/4

        print "Overlap Matrix for Tversky calculation"
        print tversky_matrix
        
        if (options.color):
            print "Aromatic"
            print aromatic_tversky_matrix
            print "Hydrogen Bond Acceptor"
            print hbondAcc_tversky_matrix
            print "Hydrogen Bond Donor"
            print hbondDon_tversky_matrix
            
        print "\n"
        
        if (options.csv):
            numpy.savetxt(os.getcwd()+'/POVME_Tversky_matrix.csv',tversky_matrix,delimiter=',')
        
        if (options.color) and (options.csv):
            numpy.savetxt(os.getcwd()+'/POVME_Tversky_matrix_aromatic.csv',aromatic_tversky_matrix,delimiter=',')        
            numpy.savetxt(os.getcwd()+'/POVME_Tversky_matrix_hbondAcceptor.csv',hbondAcc_tversky_matrix,delimiter=',')        
            numpy.savetxt(os.getcwd()+'/POVME_Tversky_matrix_hbondDonor.csv',hbondDon_tversky_matrix,delimiter=',')
            numpy.savetxt(os.getcwd()+'/POVME_Tversky_matrix_colored.csv',colored_tversky_matrix,delimiter=',')
        
        
        numpy.save('tversky_matrix.npy', tversky_matrix)
        
        if (options.color):
            numpy.save('aromatic_tversky_matrix.npy', aromatic_tversky_matrix)
            numpy.save('hbondAcc_tversky_matrix.npy', hbondAcc_tversky_matrix)
            numpy.save('hbondDon_tversky_matrix.npy', hbondDon_tversky_matrix)
                
        #print "Map of index numbers to npy files"
        #print frames_dict

            
if __name__ == "__main__": main(sys.argv)



import pylab
import numpy as np
import re

from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Draw
import os
import glob



def fixPdbSdfFormatting(data):
    datasp = data.split('\n')
    readingAtoms = True
    for lineInd, line in enumerate(datasp):
        linesp = line.split()
        if lineInd == 3:
            datasp[lineInd] = line[:16] + ' 0  0  0  0  0' +  line[30:]
        if (lineInd > 3):
            if len(linesp) != 16:
                readingAtoms = False
            if readingAtoms:
                atom = line[32:34].strip()
                datasp[lineInd] = line[:31] + atom.ljust(4) + '  ' + line[35:]
    return '\n'.join(datasp)



data = [i.split() for i in open('cluster_members.csv').readlines()]

print [len(i) for i in data]

nClusters = len(data)
nCols = 3
nRows = (nClusters+(nCols-1))/nCols
print nClusters, nRows, nCols

def getGroupPrefix(runPrefix):
    groupPrefix = runPrefix.split('run')[0].strip('_')
    return groupPrefix


groupPrefix2prefixes2frames = {}
prefix2frames = {}
for cluster in data:
    for member in cluster:
        result = re.findall('([a-zA-Z0-9_-]+)_([0-9]+)', member)
        #print member, result
        prefix = result[0][0].strip('_')
        groupPrefix = getGroupPrefix(prefix)
        if not groupPrefix in groupPrefix2prefixes2frames.keys():
            groupPrefix2prefixes2frames[groupPrefix] = {}
        groupPrefix2prefixes2frames[groupPrefix][prefix] = []
        frame = int(result[0][1])
        prefix2frames[prefix] = prefix2frames.get(prefix,[]) + [frame]
        groupPrefix2prefixes2frames[groupPrefix][prefix] += [frame]
#print prefix2frames

prefixes = prefix2frames.keys()
prefixes.sort()
nPrefixes = len(prefixes)
groupPrefixes = list(set([getGroupPrefix(i) for i in prefixes]))
groupPrefixes.sort()
nGroupPrefixes = len(groupPrefixes)
groupPrefix2row = dict([(groupPrefix, index) for index, groupPrefix in enumerate(groupPrefixes)])
#prefix2row = dict([(prefix, index) for index, prefix in enumerate(prefixes)])
prefix2row = dict([(prefix, groupPrefix2row[getGroupPrefix(prefix)]) for prefix in prefixes])

#frames = {}
prefixAndFrame2Col = {}
groupPrefixColsTaken = {}
for prefix in prefixes:
    groupPrefix = getGroupPrefix(prefix)
    frames = prefix2frames[prefix]
    frames.sort()
    #if prefix in prefixAndFrame2Col.keys():
    if groupPrefix in groupPrefixColsTaken.keys():
        #Note that frame numbering starts at 1 so off-by-1 won't be a problem here
        offset = max(groupPrefixColsTaken[groupPrefix])
    else:
        offset = 0
        
    prefixAndFrame2Col[prefix] = dict([(frame, index+offset) for index, frame in enumerate(frames)])
    groupPrefixColsTaken[groupPrefix] = groupPrefixColsTaken.get(groupPrefix,[])+[i+offset+1 for i, frame in enumerate(frames)]

#memberships = np.zeros((nClusters, nPrefixes, max([len(i) for i in prefix2frames.values()])))
#memberships = np.zeros((nClusters, nGroupPrefixes, max([len(i) for i in prefix2frames.values()])))
memberships = np.zeros((nClusters, nGroupPrefixes, max([max(i) for i in groupPrefixColsTaken.values()])))
print [(groupPrefix, max(i)) for groupPrefix, i in zip(groupPrefixColsTaken.keys(), groupPrefixColsTaken.values())]
#1/0
memberships -= 1
for clusterIndex, cluster in enumerate(data):
    for member in cluster:
        result = re.findall('([a-zA-Z0-9_-]+)_([0-9]+)', member)
        prefix = result[0][0].strip('_')
        frame = int(result[0][1])
        
        row = prefix2row[prefix]
        col = prefixAndFrame2Col[prefix][frame]
        #print member
        if sum(memberships[:,row,col]) > 0:
            print memberships[:,row,col]
            1/0
        memberships[:,row,col] = 0
        memberships[clusterIndex,row,col] = 1
        

    pylab.subplot(nRows, nCols, clusterIndex+1)
    pylab.imshow(memberships[clusterIndex,:,:],
                 interpolation='nearest',
                 vmin=-1, vmax=1,
                 aspect='auto')
    #pylab.yticks(range(len(prefixes)), [[]*4+[prefix] for i, prefix in enumerate(prefixes) if i%5==0])
    #pylab.yticks(range(0,len(prefixes),1), prefixes )
    pylab.yticks(range(0,len(groupPrefixes),1), groupPrefixes )
    pylab.xlabel('Frame')
    pylab.title('Cluster %i' %(clusterIndex))
    


pylab.subplots_adjust(left=0.06,
                      right=0.99,
                      bottom=0.025,
                      top=0.975,
                      hspace=0.250)

pylab.show()
                 
        
#Analyze inter-cluster transitions

#nSimulations = len(prefixes)
nSimulations = memberships.shape[0]
nCols = 3
# First +1 because the first plot is of all the simulations summed up
nRows = (nSimulations+1+(nCols-1))/nCols

allTransitions = np.zeros((nClusters, nClusters))
#transitions = dict(((prefix,np.zeros((nClusters, nClusters))) for prefix in prefixes))
transitions = dict(((groupPrefix,np.zeros((nClusters, nClusters))) for groupPrefix in groupPrefixes))
for simulationIndex, groupPrefix in enumerate(groupPrefixes):
    simulationIndex = groupPrefixes.index(groupPrefix)
    for prefix in groupPrefix2prefixes2frames[groupPrefix].keys():
        currentCluster = None
        sliceStartCol = min(prefixAndFrame2Col[prefix].values())
        sliceEndCol = max(prefixAndFrame2Col[prefix].values())
        #simSlice = memberships[:,simulationIndex,:]
        #for frameIndex in range(simSlice.shape[1]):
        for frameIndex in range(sliceStartCol, sliceEndCol+1):
            nonzeros = np.nonzero(memberships[:,simulationIndex,frameIndex]==1)
            if len(nonzeros[0]) == 0:
                nextCluster = None
            elif len(nonzeros[0]) > 1:
                print nextCluster
                1/0
            else:
                nextCluster = nonzeros[0][0]
            #if not(currentCluster is None) and (nextCluster is None):
            #    print prefix, 'CurrentCluster is %r and nextCluster is %r' %(currentCluster, nextCluster)
            if not(currentCluster is None) and not(nextCluster is None):
                allTransitions[currentCluster, nextCluster] += 1
                transitions[groupPrefix][currentCluster, nextCluster] += 1
                #if currentCluster == 7:
                #    print prefix, 'currentCluster is 7 and nextCluster is %r' %(nextCluster)
            currentCluster = nextCluster
            #print nextCluster
    #pylab.subplot(nRows, nCols, simulationIndex+1)
    #pylab.imshow(np.log10(transitions[prefix]),
    #             interpolation='nearest')
    #pylab.colorbar()
#pylab.show()


# Network diagram of cluster transitions

import networkx as nx
G=nx.Graph()
#nonzeros = np.nonzero(allTransitions)
#print nonzeros
#for row, col in zip(nonzeros[0], nonzeros[1]): 
nCols = 5
nRows = ((nGroupPrefixes) / nCols) 
#pylab.subplot(nRows, nCols, 1)
pylab.subplot(1, 1, 1)

for row in range(nClusters):
    for col in range(nClusters):
        G.add_edge(row,col,weight=allTransitions[row,col]+allTransitions[col,row])

 
elarge=[(u,v) for (u,v,d) in G.edges(data=True) if d['weight'] >5]
emed=[(u,v) for (u,v,d) in G.edges(data=True) if d['weight'] <=5 and d['weight'] > 2.5]
esmall=[(u,v) for (u,v,d) in G.edges(data=True) if d['weight'] <=2.5 and d['weight'] > 0]


#pos = nx.spring_layout(G, k=2./np.sqrt(nClusters))
pos = nx.spring_layout(G, k=3./np.sqrt(nClusters))

#nodes

nFramesPerClusterAllSims = [np.sum(memberships[clusterIndex,:,:]==1) for clusterIndex in range(nClusters)]
nodeSizes = [150. * float(i)/(np.mean(nFramesPerClusterAllSims)) for i in nFramesPerClusterAllSims]
nx.draw_networkx_nodes(G, pos, node_size=nodeSizes)

# edges
nx.draw_networkx_edges(G,pos,edgelist=elarge,
                    width=6)
nx.draw_networkx_edges(G,pos,edgelist=emed,
                       width=6,alpha=1.,edge_color='b',style='dashed')
nx.draw_networkx_edges(G,pos,edgelist=esmall,
                       width=6,alpha=0.3,edge_color='b',style='dashed')

# labels

nx.draw_networkx_labels(G,pos,font_size=20,font_family='sans-serif')
pylab.title('All')
pylab.xticks([],[])
pylab.yticks([],[])
pylab.show()

for simulationIndex, groupPrefix in enumerate(groupPrefixes):
    pylab.subplot(nRows, nCols, simulationIndex+1)
    G=nx.Graph()
    #nonzeros = np.nonzero(transitions[prefix])
    #for row, col in zip(nonzeros[0], nonzeros[1]):
    for row in range(nClusters):
        for col in range(nClusters):
            G.add_edge(row,col,weight=transitions[groupPrefix][row,col]+transitions[groupPrefix][col,row])


    elarge=[(u,v) for (u,v,d) in G.edges(data=True) if d['weight'] >5]
    emed=[(u,v) for (u,v,d) in G.edges(data=True) if d['weight'] <=5 and d['weight'] > 2.5]
    esmall=[(u,v) for (u,v,d) in G.edges(data=True) if d['weight'] <=2.5 and d['weight'] > 0]

    #pos = nx.spring_layout(G)

    # nodes
    nFramesPerClusterThisSim = [np.sum(memberships[clusterIndex,simulationIndex,:]==1) for clusterIndex in range(nClusters)]
    print nFramesPerClusterThisSim
    nodeSizes = [150. * float(i)/(np.mean(nFramesPerClusterAllSims)/nGroupPrefixes) for i in nFramesPerClusterThisSim]
    print nodeSizes
    nx.draw_networkx_nodes(G, pos, node_size=nodeSizes)

    
    # edges
    nx.draw_networkx_edges(G,pos,edgelist=elarge,
                           width=6)
    nx.draw_networkx_edges(G,pos,edgelist=emed,
                           width=6,alpha=1.,edge_color='b',style='dashed')
    nx.draw_networkx_edges(G,pos,edgelist=esmall,
                           width=6,alpha=0.3,edge_color='b',style='dashed')

    # labels
    nx.draw_networkx_labels(G,pos,font_size=20,font_family='sans-serif')
    
    pylab.xlim([-0.2, 1.2])
    pylab.ylim([-0.2, 1.2])
    pylab.title(groupPrefix)
    pylab.xticks([],[])
    pylab.yticks([],[])
pylab.show()

#print transitions
## Get simulation similarities by cluster vector overlap
groupClusterVecSimilarities = np.zeros((nGroupPrefixes,nGroupPrefixes))
for index1, groupPrefix1 in enumerate(groupPrefixes):
    
    clusterPops1 = np.diag(transitions[groupPrefix1])
    clusterVect1 = clusterPops1 / np.linalg.norm(clusterPops1)
    for index2, groupPrefix2 in enumerate(groupPrefixes):
        clusterPops2 = np.diag(transitions[groupPrefix2])
        clusterVect2 = clusterPops2 / np.linalg.norm(clusterPops2)
        groupClusterVecSimilarities[index1, index2] = np.dot(clusterVect1,clusterVect2)


## Get simulation similarities by average tanimoto score
index2frameData = open('indexMapToFrames.csv').readlines()

prefix2indices = {}
index2prefix = {}
for line in index2frameData:
    index = line.split(',')[0]
    prefix = line.split('/')[-1].split('_')[0]
    prefix2indices[prefix] = prefix2indices.get(prefix,[]) + [int(index)]
    index2prefix[index] = prefix

sortedPrefixes = prefix2indices.keys()
sortedPrefixes.sort()
groupOverlapSimilarities = np.zeros((nGroupPrefixes, nGroupPrefixes))
tanimoto = np.load('tanimoto_matrix.npy')
for index1, prefix1 in enumerate(sortedPrefixes):
    for index2, prefix2 in enumerate(sortedPrefixes):
        if index2 < index1:
            continue
        indices1 = np.array(prefix2indices[prefix1])
        #print indices1
        indices2 = np.array(prefix2indices[prefix2])
        avSimSimilarity = np.mean(tanimoto[indices1, indices2])
        groupOverlapSimilarities[index1, index2] = avSimSimilarity
        groupOverlapSimilarities[index2, index1] = avSimSimilarity


pylab.subplot(1,3,2)

pylab.imshow(groupClusterVecSimilarities,
             interpolation='nearest',
             vmin=0., vmax=1.,
             origin='lower',
             extent=(-0.5,
                      0.5+groupClusterVecSimilarities.shape[0],
                      -0.5,
                      0.5+groupClusterVecSimilarities.shape[1])
             )

pylab.title('Normalized Cluster Vector Overlap')
maxTicLoc = len(groupPrefixes)+1.
ticLocs = np.arange(0, maxTicLoc,maxTicLoc/(len(groupPrefixes))) 
pylab.xticks(ticLocs, groupPrefixes, rotation=90)
pylab.yticks(ticLocs, groupPrefixes)
#pylab.xticks(np.arange(len(groupPrefixes))+0.5, groupPrefixes)
#pylab.yticks(np.arange(len(groupPrefixes))+0.5, groupPrefixes)
#pylab.colorbar()
#pylab.show()

pylab.subplot(1,3,1)
pylab.imshow(groupOverlapSimilarities,
             interpolation='nearest',
             vmin=0., vmax=1.,
             origin='lower',
             extent=(-0.5,
                      0.5+groupOverlapSimilarities.shape[0],
                      -0.5,
                      0.5+groupOverlapSimilarities.shape[1])
             )

pylab.title('Average Simulation Tanimoto')
maxTicLoc = len(groupPrefixes)+1.
ticLocs = np.arange(0, maxTicLoc,maxTicLoc/(len(groupPrefixes))) 
pylab.xticks(ticLocs, groupPrefixes, rotation=90)
pylab.yticks(ticLocs, groupPrefixes)
#pylab.xticks(range(len(groupPrefixes)), groupPrefixes)
#pylab.yticks(range(len(groupPrefixes)), groupPrefixes)
#pylab.colorbar()
#pylab.show()

pdb2Lig = {#'1BYQ':'ADP',
           '1UYF':'PU1',
           '1UYI':'PUZ',
           '1UYL':'NOLIG',
           #'2VCI':'2GJ',
           #'2WI7':'2KL',
           #'2XHR':'C0P',
           #'2YEJ':'ZZ3',
           #'2YEJ':'XQK',
           #'3B26':'B2L',
           '3D0B':'SNX',
           #'3HEK':'BD0',
           #'3K98':'1RC',
           #'3K99':'PFT',
           #'3RKZ':'06T', 
           #'3RLR':'3RR',
           #'4CWN':'6LV',
           #'4FCR':'0TM',
           #'4LWE':'FJ2',
           '4R3M':'JR9',
           #'4W7T':'3JC'
           } 

pdb2LigMol = {}
import urllib2
sortedPdbList = pdb2Lig.keys()
sortedPdbList.sort()
for pdb in sortedPdbList:
    print pdb, pdb2Lig[pdb]
    if pdb2Lig[pdb] == 'NOLIG':
        pdb2LigMol[pdb] = Chem.Mol()
        continue
    url = 'http://www.rcsb.org/pdb/download/downloadLigandFiles.do?ligandIdList=%s&structIdList=%s&instanceType=all&excludeUnobserved=false&includeHydrogens=false' %(pdb2Lig[pdb],pdb)
    #print url
    response = urllib2.urlopen(url)
    #print "Response received"
    import time
    time.sleep(0.5)
    data = response.read()
    #with open('temp_orig.sdf','wb') as of:
    #    of.write(data)
    data = fixPdbSdfFormatting(data)
    #with open('temp.sdf','wb') as of:
    #    of.write(data)
    #os.system('babel -isdf temp.sdf -x3 -osdf  temp2.sdf >& xx')
    #stio = StringIO.StringIO(data)
    #print data
        
    #suppl = Chem.SDMolSupplier('temp.sdf')

    suppl = Chem.SDMolSupplier()
    suppl.SetData(data)
    for mol in suppl:
        #smiles = Chem.MolToSmiles(mol)
        #print smiles
        m_mol = Chem.MolFromSmiles(mol.GetProp('SMILES'))
        AllChem.Compute2DCoords(m_mol)
        pdb2LigMol[pdb] = Chem.Mol(m_mol)
        break #Only read first mol


legends = []
molList = []

sortedPdbList2 = pdb2LigMol.keys()
sortedPdbList2.sort()
for pdb in sortedPdbList2:
    legends.append(pdb)
    molList.append(pdb2LigMol[pdb])


    
#>>> m1 = Chem.MolFromSmiles('Cc1ccccc1')
#>>> fp1 = AllChem.GetMorganFingerprint(m1,2)
#>>> fp1
#<rdkit.DataStructs.cDataStructs.UIntSparseIntVect object at 0x...>
#>>> m2 = Chem.MolFromSmiles('Cc1ncccc1')
#>>> fp2 = AllChem.GetMorganFingerprint(m2,2)
#>>> DataStructs.DiceSimilarity(fp1,fp2)

from rdkit import DataStructs
from rdkit.Chem.Fingerprints import FingerprintMols
molSimilarities = np.zeros((len(sortedPdbList2), len(sortedPdbList2)))
morganFingerprints = False
for idx1, pdb1 in enumerate(sortedPdbList2):
    if morganFingerprints:
        molFp1 = AllChem.GetMorganFingerprint(molList[idx1],3, useFeatures=True)
    else:
        molFp1 = FingerprintMols.FingerprintMol(molList[idx1])
        
    for idx2, pdb2 in enumerate(sortedPdbList2):
        if morganFingerprints:
            molFp2 = AllChem.GetMorganFingerprint(molList[idx2],3, useFeatures=True)
            sim = DataStructs.DiceSimilarity(molFp1, molFp2)
        else:
            molFp2 = FingerprintMols.FingerprintMol(molList[idx2])
            sim = DataStructs.FingerprintSimilarity(molFp1, molFp2)
        molSimilarities[idx1,idx2] = sim

pylab.subplot(1,3,3)
pylab.imshow(molSimilarities,
             interpolation='nearest',
             vmin=0., vmax=1.,
             origin='lower',
             extent=(-0.5,
                      0.5+molSimilarities.shape[0],
                      -0.5,
                      0.5+molSimilarities.shape[1])
             )

pylab.title('Ligand Similarity')
maxTicLoc = len(groupPrefixes)+1.
ticLocs = np.arange(0, maxTicLoc,maxTicLoc/(len(groupPrefixes))) 
pylab.xticks(ticLocs, groupPrefixes, rotation=90)
pylab.yticks(ticLocs, groupPrefixes)
#pylab.xticks(range(len(legends)), legends)
#pylab.yticks(range(len(legends)), legends)
#pylab.colorbar()
pylab.show()

flatGroupOverlapSimilarities = groupOverlapSimilarities[np.triu_indices(nGroupPrefixes,1)]
flatMolSimilarities = molSimilarities[np.triu_indices(nGroupPrefixes,1)]

import random
import scipy.stats
print "Calculating Rank Correlations"
actualTau = scipy.stats.kendalltau(flatGroupOverlapSimilarities,
                                  flatMolSimilarities)
print "Actual Kendall Tau:", actualTau
actualRho = scipy.stats.spearmanr(flatGroupOverlapSimilarities,
                                  flatMolSimilarities)
print "Actual Spearman's Rho:", actualRho
nResamplings = 10000
bootstrapRhos = []
bootstrapTaus = []
#flatGroupOverlapSimilarities = np.random.random(flatGroupOverlapSimilarities.shape)
#flatMolSimilarities = np.random.random(flatMolSimilarities.shape)
for i in range(nResamplings):
    newOrder = range(len(flatMolSimilarities))
    random.shuffle(newOrder)
    scrambledMolSimilarities = [flatMolSimilarities[i] for i in newOrder]
    scrambledRho = scipy.stats.spearmanr(flatGroupOverlapSimilarities,
                                         scrambledMolSimilarities)
    bootstrapRhos.append(scrambledRho[0])
    scrambledTau = scipy.stats.kendalltau(flatGroupOverlapSimilarities,
                                          scrambledMolSimilarities)
    bootstrapTaus.append(scrambledTau[0])
    
nBins = 50
pylab.subplot(1,2,1)
hist = pylab.hist(bootstrapRhos,
                  bins=nBins,
                  cumulative=True,
                  histtype='step')
pylab.axvline(actualRho[0], 0, 1,
              color='r',
              linewidth=4)
pylab.title("Spearman's Rho Bootstrap")
pylab.xlabel("Spearman's Rho")
pylab.ylabel('Population')

pylab.subplot(1,2,2)
hist = pylab.hist(bootstrapTaus,
                  bins=nBins,
                  cumulative=True,
                  histtype='step')
pylab.axvline(actualTau[0], 0, 1,
              color='r',
              linewidth=4)
pylab.title("Kendall Tau Bootstrap")
pylab.xlabel("Kendall Tau")
pylab.ylabel('Population')
pylab.show()                     

'''
#Show cluster representatives
showFirstNClusters = 6
nAngles = 1

nCols = 4
nRows = (showFirstNClusters / 2) + 1
import matplotlib.image as mpimg
import os
#for clusterNum in range(nClusters):
for clusterNum in range(showFirstNClusters):
    thisRow = (clusterNum / 2) + 1
    thisCol = ((nAngles+1) * (clusterNum % 2))+1
    pylab.subplot(nRows,nCols, ((thisRow-1)*nCols)+thisCol)
    pylab.text(0,0,'Cluster %i'%(clusterNum), fontsize=30)
    #for angle in range(1,5):
    for angle in range(1,nAngles+1):
        if not(os.path.exists('clusterPics/cluster%i_render%i.png')):
            os.system('convert clusterPics/cluster%i_render%i.tga clusterPics/cluster%i_render%i.png' %(clusterNum, angle, clusterNum, angle))
        
        img = mpimg.imread('clusterPics/cluster%i_render%i.png' %(clusterNum, angle))
        thisCol = ((nAngles+1) * (clusterNum % 2)) + angle + 1
        pylab.subplot(nRows, nCols,((thisRow-1)*nCols)+thisCol)
        pylab.imshow(img)
        pylab.xticks([],[])
        pylab.yticks([],[])
pylab.show()
'''



#!python
# Implementation of Clustering Algorithms in POVME
# By Celia Wong
# Advised by Jeff Wagner
# Amaro Lab, UCSD

import scipy.cluster.vq, scipy.cluster.hierarchy
import argparse
import numpy
import sys
import os
import csv
import copy
import itertools
import collections
#import fnmatch
import pylab
import POVME.packages.binana.peel as peel
#import matplotlib.pyplot

class InputReader():

    def __init__(self):
        #self.coordinates = []
        #self.frames = 0
        self.overlapMatrix = []
        self.prefixToTrajectory = {}
        self.indexToNpyFile = {}
        self.indexToFrame = {}
        self.indexToPrefix = {}

    # Save each frame in the trajectory as a set
    ''' def read_traj(self,traj_file):

        trajectory = open(traj_file,'r')
        frame_coordinates = []
#        temp_coordinates = numoy.array([])

        for line in trajectory:
            if line[0] == 'E':
                self.coordinates.append(frame_coordinates)

#                numpy.append(self.coordinates,frame_coordinates)
                frame_coordinates = []

                self.frames += 1

            elif line[0] == 'A':


            #output_file_name = 'hierarchical_' + command_input['output_name'] + '.csv'

                if line[17] != 'X':
                    frame_coordinates.append((float(line[29:37].strip()),float(line[38:45].strip()),float(line[46:54].strip())))
                    #numpy.append(frame_coordinates,(float(line[29:37].strip()),float(line[38:45].strip()),float(line[46:54].strip())))


        self.coordinates = numpy.array(self.coordinates)

        trajectory.close()
    '''
        
    def read_indexFile(self,indexToFrameFile):
        if indexToFrameFile == None:
            return
        #self.indexToNpyFile = {}
        #self.indexToFrame = {}
        with open(indexToFrameFile) as csvfile:
            fieldnames = ['index','frameFile']
            reader = csv.DictReader(csvfile, fieldnames=fieldnames)
            for row in reader:
                self.indexToNpyFile[int(row['index'])] = row['frameFile']
                if self.indexToFrame != None:
                    try:
                        frameNumber = row['frameFile'].split('frame_')[-1].replace('.npy','')
                        self.indexToFrame[int(row['index'])] = int(frameNumber)
                        framePrefix = row['frameFile'].split('/')[-1].replace('frame_%s.npy'%(frameNumber),'')
                        self.indexToPrefix[int(row['index'])] = framePrefix
                    except:
                        raise Exception("Unable to strip frame number or prefix from input filename %s. Disabling frame number output." %(row['frameFile']))
                        #self.indexToFrame = None
                        #self.indexToPrefix = None
                #self.coordinates.append(numpy.load(row['frame']))
        

    def read_overlap(self,overlap_file):
        overlap_suffix = overlap_file[-4:]
        if overlap_suffix == '.npy':
            self.overlapMatrix = numpy.load(open(overlap_file))
        elif overlap_suffix[-4:] == '.csv':
            overlap = open(overlap_file,'r')
            overlap_values = csv.reader(overlap, delimiter = ',')
    
            for line in overlap_values:
                #self.overlapMatrix.append([float(x) for x in line])
                row = []
                #self.frames += 1
                for value in line:
                    row.append(float(value))
                self.overlapMatrix.append(row)
    
            self.overlapMatrix = numpy.array(self.overlapMatrix)
            #self.oneMinusOverlapMatrix = 1. - numpy.array(self.overlapMatrix)
    
            overlap.close()
        else:
            raise Exception('Unrecognized overlap matrix input file type:', overlap_suffix)

    
    def parse_traj_inputs(self, argst, argsT):
        if argsT != None:
            for argT in argsT:
                data = open(argT).read()
                datasp = data.strip().split()
                for line in datasp:
                    linesp = line.split(':')
                    prefix = linesp[0].strip()
                    trajFile = linesp[1].strip()
                    self.prefixToTrajectory[prefix] = trajFile
        for argt in argst:
            argtsp = argt.split(':')
            prefix = argtsp[0].strip()
            trajFile = argtsp[1].strip()
            self.prefixToTrajectory[prefix] = trajFile
        ## Check to ensure these files exist
        for prefix in self.prefixToTrajectory:
            trajFileName = self.prefixToTrajectory[prefix]
            if not(os.path.exists(trajFileName)):
                raise Exception('ERROR - trajectory file %s doesn\'t exist!' %(trajFileName))

                

class Cluster():
    #def __init__(self,coordinates,frames,overlap_values,frameToFileName):
    def __init__(self,input_reader):
        #self.coordinates = coordinates
        #self.frames = frames
        self.overlap_values = input_reader.overlapMatrix
        self.one_minus_overlap_values = 1. - self.overlap_values
        self.frames = len(self.overlap_values)
        self.indexToFrame = input_reader.indexToFrame
        self.indexToPrefix = input_reader.indexToPrefix
        self.indexToNpyFile = input_reader.indexToNpyFile
        self.prefixToTrajectory = input_reader.prefixToTrajectory
        self.avgLinkage = None
        self.whited_overlap_values = None

        self.do_sanity_checks()

    def do_sanity_checks(self):
        self.check_commandline_inputs()
        self.ensure_file_prefixes_map_to_trajectories()
        #self.ensure_trajectories_exist() #Check performed during -T argument parsing instead
        
    def check_commandline_inputs(self):
        if (self.indexToNpyFile == {}) and (self.prefixToTrajectory != {}):
            raise Exception("ERROR! Given pdb trajectory (-t/T) but not given index file (-i). Output will return matrix indices instead of frame numbers or cluster representative structures.")
        elif self.indexToNpyFile == {}:
            print "Not given index file (-i). Clustering will return matrix indices, but not trajectory frame numbers or members/representatives."
        elif (self.indexToNpyFile != {}) and(self.prefixToTrajectory == {}):
            print "Given index file (-i) but not prefix-to-trajectory mapping (-t or -T). Clustering will return prefix and frame numbers of cluster members, but will not extract representatives."
        elif (self.indexToNpyFile != {}) and (self.prefixToTrajectory != {}):
            print "Given index file (-i) and prefix-to-trajectory mapping (-t or -T). Clustering will return prefix and frame numbers of cluster members, and will extract representatives."

    def ensure_file_prefixes_map_to_trajectories(self):
        if (self.prefixToTrajectory == {}) or (self.indexToNpyFile == {}):
            print "No -i and/or -t/T arguments given - Skipping file-prefix-to-trajectory mapping completeness test"
            return
        else:        
            allPrefixesSet = set(self.indexToPrefix.values())
            for prefix in allPrefixesSet:
                if not(prefix in self.prefixToTrajectory.keys()):
                    raise Exception('File prefix %s not found in -t arguments (which are %r)' %(prefix, self.prefixToTrajectory.keys()))
        return
        
    def ensure_trajectories_exist(self):
        if self.prefixToTrajectory == {}:
            print "No -t/T arguments given. Skipping trajectory-file-existence check"
        else:
            for trajectoryFile in self.prefixToTrajectory.values():
                if not(os.path.exists(trajectoryFile)):
                    raise Exception("Trajectory file %s not found" %(trajectoryFile))
            
            
    def kmeans_cluster(self,number_clusters):

        if self.whited_overlap_values == None:
            self.whited_overlap_values = scipy.cluster.vq.whiten(self.overlap_values)
        frames,result = scipy.cluster.vq.kmeans(self.whited_overlap_values, number_clusters)
        #frames,result = scipy.cluster.vq.kmeans2(self.whited_overlap_values, number_clusters)
        code, dist = scipy.cluster.vq.vq(self.whited_overlap_values,frames)

        print "The clusters are {0}".format(code)
        print code.shape
        list_of_clusters = self.separate_clusters(code)
        #return code
        return list_of_clusters

    def hierarchical_cluster(self, number_clusters):
        if self.avgLinkage == None:
            try:
                overlapHash = str(numpy.sum(self.one_minus_overlap_values.flatten()[::self.one_minus_overlap_values.size/100]))[-7:]
            except:
                overlapHash = str(numpy.sum(self.one_minus_overlap_values.flatten()))[-7:]
            linkageFile = 'avg_linkage_hash_%s.npy' %(overlapHash)
            if os.path.exists(linkageFile):
                self.avgLinkage = numpy.load(linkageFile)
            else:
                self.avgLinkage = scipy.cluster.hierarchy.average(self.one_minus_overlap_values)
                numpy.save(linkageFile, self.avgLinkage)
        result = scipy.cluster.hierarchy.fcluster(self.avgLinkage,
                                                  number_clusters,
                                                  criterion='maxclust')
        
        #result = scipy.cluster.hierarchy.linkage(self.overlap_values)
        # Hierarchical clustering seems to have a bad habit of returning clusters nubered starting from 1 instead of 0. This is undesired behavior.
        result = result - 1
        clusters = self.separate_clusters(result)
        return clusters
    
#        scipy.cluster.hierarchy.dendrogram(result)


    ''' separate_cluster_traj will seperate the original trajectory into the
        number of clusters specified. Each new cluster traj will only contain the
        frames that belong in that cluster.

        cluster_result = list that is number of frames long and contain the cluster
                         each frame is grouped with.
        number_clusters = the number of clusters specified.
        file_name = the file name that was passed into main() as a command line arg.
        traj_file = the original trajectory file containing all frames
    '''
    def separate_cluster_traj(self,cluster_result,number_clusters,file_name,traj_file,output_file):

        list_writes = [None]*number_clusters

        '''Opening n number of clusters (n = number of clusters previously indicated)'''
        for i in range(number_clusters):
            list_writes[i] = open('cluster_'+ str(i)+'.pdb','wb')

        initial_pdb = open(traj_file,'rb')

        current_frame = 0
        current_cluster = cluster_result[current_frame]

        for line in initial_pdb:

            if line[0] == 'E':
                list_writes[current_cluster].write(line)
                if current_frame < len(cluster_result)-1:
                    current_frame += 1
                    current_cluster = cluster_result[current_frame]
            else:
                list_writes[current_cluster].write(line)

        initial_pdb.close()
        for i in range(number_clusters):
            list_writes[i].close()

    ''' Separates the frames into the set clusters
    '''

    def separate_clusters(self,cluster_results):

#        print "cluster results: {0}".format(cluster_results)
        total_num_clusters = len(set(cluster_results))
        list_clusters = [list([]) for i in xrange(total_num_clusters)]

        for i in range(len(cluster_results)):
#            print "Cluster_res for {0} is {1}".format(i, cluster_results[i])
            list_clusters[cluster_results[i]].append(i)
        list_clusters.sort(key=len, reverse=True)
        return list_clusters

    ''' csv file containing differences in binding site is first argument already read and stored into memory by previous command '''

    #def find_centroids(self,binding_volume_matrix,cluster_results,number_clusters,number_frames, indexToFrame):
    def find_centroids(self, list_of_clusters, outputPrefix):
        #number_clusters = len(set(cluster_results))
        #list_of_clusters = self.separate_clusters(cluster_results)
        #print list_of_clusters

        ''' set to some arbitrary large number? '''
        #shortest_average_distance = [1.e20] * number_clusters

        #centroid_list = [[0] for i in xrange(number_clusters)]
        centroid_list = []
        for cluster in list_of_clusters:
            sum_distances = []
            if len(cluster) == 1:
                sum_distances.append(0)
            else:
                cluster = numpy.array(cluster)
                for entry in cluster:
                    allButEntry = cluster[cluster != entry]
                    #print cluster, entry, allButEntry
                    #print self.one_minus_overlap_values[entry,:]
                    totalDist = numpy.sum(self.one_minus_overlap_values[entry,allButEntry])
                    sum_distances.append(totalDist)
                
            #print cluster, sum_distances, numpy.argsort(sum_distances)[0]
            centroid_cluster_index = numpy.argsort(sum_distances)[0]
            centroid_global_index = cluster[centroid_cluster_index]
            centroid_list.append(centroid_global_index)
                
            


        
        if (self.indexToFrame == {}) and (self.indexToNpyFile != {}):
            repsFileName = '%scluster_reps.csv' %(outputPrefix)
            membersFileName = '%scluster_members.csv' %(outputPrefix)
            print "Unable to extract frame numbers from file names. Writing out npy file names to %s and %s" %(repsFileName, membersFileName)
            with open(repsFileName,'wb') as of:
                cluster_rep_file_names = [str(self.indexToNpyFile[i]) for i in centroid_list]
                of.write('\n'.join(cluster_rep_file_names))
            with open(membersFileName,'wb') as of:
                for cluster in list_of_clusters:
                    cluster_member_file_names =[str(self.indexToNpyFile[i]) for i in cluster] 
                    of.write(' '.join(cluster_member_file_names))
                    of.write('\n')
            
        elif (self.indexToFrame == {}) and (self.indexToNpyFile == {}):
            print "No matrix-index-to-trajectory-frame mapping given. Writing out matrix indices"
            with open('%scluster_reps.csv' %(outputPrefix),'wb') as of:
                of.write('\n'.join([str(i) for i in centroid_list]))
            with open('%scluster_members.csv' %(outputPrefix),'wb') as of:
                for cluster in list_of_clusters:
                    of.write(' '.join([str(i) for i in cluster]))
                    of.write('\n')
                    
        elif (self.indexToFrame != {}):
            repsFileName = '%scluster_reps.csv' %(outputPrefix)
            membersFileName = '%scluster_members.csv' %(outputPrefix)
            print "Matrix-index-to-trajectory-frame mapping given. Writing out trajectory frames to %s and %s." %(repsFileName, membersFileName)
            
            with open(repsFileName,'wb') as of:
                cluster_rep_frame_nums = [str(self.indexToFrame[i]) for i in centroid_list]
                cluster_rep_prefixes = [str(self.indexToPrefix[i]) for i in centroid_list]
                cluster_rep_strings = ['_'.join(i) for i in zip(cluster_rep_prefixes, cluster_rep_frame_nums)]
                of.write('\n'.join(cluster_rep_strings))
            with open(membersFileName,'wb') as of:
                for cluster in list_of_clusters:
                    cluster_member_frame_nums =[str(self.indexToFrame[i]) for i in cluster]
                    cluster_member_prefixes = [str(self.indexToPrefix[i]) for i in cluster]
                    cluster_member_strings = ['_'.join(i) for i in zip(cluster_member_prefixes, cluster_member_frame_nums)]
                    of.write(' '.join(cluster_member_strings))
                    of.write('\n')

        if (self.indexToFrame != {}) and (self.prefixToTrajectory != {}):
            print "Extracting trajectory frames"
            matrixIndex2Cluster = {}
            for index, centroid in enumerate(centroid_list):
                matrixIndex2Cluster[centroid] = index
            clusterInd2CentFileName = self.extractFrames(matrixIndex2Cluster, outputPrefix, reps=True)
        else:
            clusterInd2CentFileName = {}
        return clusterInd2CentFileName
        
    def outputAllFrames(self, list_of_clusters, outputPrefix):
        ## check to make sure we'll be able to map all matrix indices to files
        for clusterInd, cluster in enumerate(list_of_clusters):
            #print cluster
            #print indexToFrame.keys()
            for matrixInd in cluster:
                if not(matrixInd in self.indexToFrame.keys()):
                    raise Exception('User requested all frame pdbs to be output to cluster directories, but the program is unable to map all overlap matrix indices to trajectory/frame combinations. Make sure that -t/-T and -i arguments cover all frames and prefixes. Index: %i Cluster: %i' %(matrixInd, clusterInd))

        ## If all mappings exist, extract all relevant frames
        matrixInd2Cluster = {}
        for clusterInd, cluster in enumerate(list_of_clusters):
            for matrixInd in cluster:
                matrixInd2Cluster[matrixInd] = clusterInd
        self.extractFrames(matrixInd2Cluster, outputPrefix, reps=False)
        
            
    
    def extractFrames(self, matrixIndex2Cluster, outputPrefix, reps=False):
        framesToExtract = {}
        clusterInd2CentFileName = {}
        for thisMatrixIndex in matrixIndex2Cluster:
            thisCluster = matrixIndex2Cluster[thisMatrixIndex]
            npyFileName = self.indexToNpyFile[thisMatrixIndex]
            npyFilePrefix = npyFileName.split('/')[-1].split('frame_')[0]
            frameNum = int(npyFileName.split('/')[-1].split('frame_')[-1].replace('.npy',''))
            prefixMatch = ''
            ## See if this prefix is in our dictionary or trajectories
            for trajPrefix in self.prefixToTrajectory.keys():
                if trajPrefix == npyFilePrefix:
                    if prefixMatch == '':
                        prefixMatch = trajPrefix
                    else: # If a matching prefix has already been found
                        raise Exception('ERROR - file %s matches prefix %s and %s' %(npyFileName, trajPrefix, prefixMatch))
            ## Disabled this block - All prefix-to-trajectory matching should be explicit. This caused an error when POVME was run whith a blank prefix
            #if prefixMatch == '':
            #    trajFileName = npyFilePrefix + '.pdb'
            #else:
            trajFileName = self.prefixToTrajectory[prefixMatch]

            ## Figure out the directory and filename that this frame should be written to    
            outputDir = '%scluster%i' %(outputPrefix, thisCluster)
            if not os.path.exists(outputDir):
                os.system('mkdir %s' %(outputDir))
            if reps == True:
                outputFileName = 'REP_%sframe_%i.pdb' %(prefixMatch, frameNum)
                clusterInd2CentFileName[thisCluster] = outputFileName
            else:
                outputFileName = '%sframe_%i.pdb' %(prefixMatch, frameNum)
            fullOutputFileName = outputDir + '/' + outputFileName
            if not trajFileName in framesToExtract.keys():
                framesToExtract[trajFileName] = {}
                
            framesToExtract[trajFileName][frameNum] = fullOutputFileName
            
        for trajFileName in framesToExtract:
            frameCounter = 1
            frameData = ''
            with open(trajFileName) as fo:
                for line in fo:
                    if frameCounter in framesToExtract[trajFileName]:
                        frameData += line
                    if 'END' == line[:3]:
                        if frameData != '':
                            thisOutputFileName = framesToExtract[trajFileName][frameCounter]
                            with open(thisOutputFileName,'wb') as of:
                                of.write(frameData)
                        frameData = ''
                        frameCounter += 1
                if frameData != '':
                    thisOutputFileName = framesToExtract[trajFileName][frameCounter]
                    with open(thisOutputFileName,'wb') as of:
                        of.write(frameData)
        return clusterInd2CentFileName
        
    '''                
    def extractFrame(self, matrixIndex, outputDir, rep=False):
        npyFileName = self.indexToNpyFile[matrixIndex]
        npyFilePrefix = npyFileName.split('/')[-1].split('frame_')[0]
        frameNum = int(npyFileName.split('/')[-1].split('frame_')[-1].replace('.npy',''))
        prefixMatch = ''
        for trajPrefix in self.prefixToTrajectory.keys():
            if trajPrefix == npyFilePrefix:
                if prefixMatch == '':
                    prefixMatch = trajPrefix
                else: # If a matching prefix has already been found
                    raise Exception('ERROR - file %s matches prefix %s and %s' %(npyFileName, trajPrefix, prefixMatch))
                    
        if prefixMatch == '':
            trajFileName = npyFilePrefix + '.pdb'
        else:
            trajFileName = self.prefixToTrajectory[prefixMatch]
            
        if rep == True:
            outputFileName = '%s/REP_%sframe_%i.pdb' %(outputDir, prefixMatch, frameNum)
        else:
            outputFileName = '%s/%sframe_%i.pdb' %(outputDir, prefixMatch, frameNum)
        frameCounter = 0
        frameData = ''
        with open(trajFileName) as fo:
            for line in fo:
                if frameCounter == frameNum:
                    frameData += line
                if 'END' in line.strip():
                    frameCounter += 1
        
        with open(outputFileName,'wb') as of:
            of.write(frameData)
            '''            
    def generate_difference_maps(self, list_of_clusters, clusterInd2CentFileName, outputPrefix):
        print "Generating difference maps"
        #nFrames = len(frame_assignments)
        nFrames = sum([len(i) for i in list_of_clusters])
        #list_of_clusters = self.separate_clusters(frame_assignments)
        #nClusters = len(list_of_clusters)
        allFrameCounts = {}
        clusterCounts = [] 
        for clusterIndex, cluster in enumerate(list_of_clusters):
            nClusterFrames = len(cluster)
            thisClusterCounts = {}
            for matrixIndex in cluster:
                npyFilename = self.indexToNpyFile[matrixIndex]
                points = numpy.load(npyFilename)
                if len(points) == 0:
                    continue
                # If the list has intensity values
                if points.shape[1]==4:
                    for point in points:
                        tuplePoint = tuple(point[:3])
                        allFrameCounts[tuplePoint] = allFrameCounts.get(tuplePoint,0) + (point[3]/nFrames)
                        thisClusterCounts[tuplePoint] = thisClusterCounts.get(tuplePoint,0) + (point[3]/nClusterFrames)
                else:
                    for point in points:
                        tuplePoint = tuple(point)
                        allFrameCounts[tuplePoint] = allFrameCounts.get(tuplePoint,0) + (1.0/nFrames)
                        thisClusterCounts[tuplePoint] = thisClusterCounts.get(tuplePoint,0) + (1.0/nClusterFrames)
                        
            clusterCounts.append(thisClusterCounts)
            
        allPoints = numpy.array(allFrameCounts.keys())
        allFrameMap = peel.featureMap.fromPovmeList(allPoints, justCoords = True, skinDistance=2.)
        allFrameMap.data[:] = 0.0
        
        for point in allFrameCounts.keys():
            thisIndex = allFrameMap.point_to_nearest_index(point)
            allFrameMap.data[thisIndex] = allFrameCounts[point]
        
        clusterMaps = []
        for thisClusterCounts in clusterCounts:
            thisClusterMap = peel.featureMap.fromPovmeList(allPoints, justCoords = True, skinDistance=2.)
            thisClusterMap.data[:] = 0.0
            for point in thisClusterCounts.keys():
                thisIndex = allFrameMap.point_to_nearest_index(point)
                thisClusterMap.data[thisIndex] = thisClusterCounts[point]
            
            clusterMaps.append(thisClusterMap)

        templateLoadDifference = '''mol new {%s} waitfor all
display projection Orthographic
mol modstyle 0 top Isosurface 0.7500000 0 0 1 1 1
#mol modstyle 0 !MOLID! Isosurface 0.2500000 0 1 1 1 1
#white
mol modcolor 0 top ColorID 8 

mol addfile {%s} waitfor all
mol addrep top
#mol addrep !MOLID!
mol modstyle 1 top Isosurface 0.250000 1 0 1 1 1
#mol modstyle 1 !MOLID! Isosurface 0.250000 1 2 1 1 1
#blue
mol modcolor 1 top ColorID 0 
mol showrep top 1 0 
#mol showrep !MOLID! 1 0 

mol addfile {%s} waitfor all
mol addrep top
#mol addrep !MOLID!
mol modmaterial 2 top Transparent
mol modstyle 2 top Isosurface 0.2500000 2 0 0 1 1
#mol modstyle 2 !MOLID! Isosurface 0.2500000 2 2 1 1 1
#green
mol modcolor 2 top ColorID 12

mol addfile {%s} waitfor all
mol addrep top
#mol addrep !MOLID!
mol modmaterial 3 top Transparent
mol modstyle 3 top Isosurface -0.7500000 3 0 0 1 1
#mol modstyle 3 !MOLID! Isosurface -0.2500000 3 2 1 1 1
#red
mol modcolor 3 top ColorID 1

# Now load the protein
mol addfile {%s} type {pdb} first 0 last -1 step 1 waitfor all
mol modstyle 4 top NewCartoon 0.300000 10.000000 4.100000 0



'''

        plotAll = ''
        
        allFrameDxName = '%saveragePocket.dx' %(outputPrefix)
        allFrameMap.write_dx_file(allFrameDxName)

        for clusterIndex, clusterMap in enumerate(clusterMaps):
            outputDir = '%scluster%i' %(outputPrefix, clusterIndex)
            if not os.path.exists(outputDir):
                os.system('mkdir %s' %(outputDir))
            thisClusterDxName = '%saverage.dx' %(outputPrefix)
            clusterMap.write_dx_file(outputDir+'/'+thisClusterDxName)
            
            differenceMap = copy.deepcopy(clusterMap)
            differenceMap.data = differenceMap.data - allFrameMap.data

            thisDifferenceDxName = '%sdifference.dx' %(outputPrefix)
            differenceMap.write_dx_file(outputDir+'/'+thisDifferenceDxName)
        
            thisCentroidPdbName = clusterInd2CentFileName[clusterIndex]
            
            thisVmdScript = templateLoadDifference %('../'+allFrameDxName, 
                                                     thisClusterDxName, 
                                                     thisDifferenceDxName, 
                                                     thisDifferenceDxName,
                                                     thisCentroidPdbName)
            thisVmdScript = thisVmdScript.replace('!MOLID!', '0')

            with open('%s/visualize.vmd' %(outputDir),'wb') as of:
                of.write(thisVmdScript)
            

            plotAllContrib = templateLoadDifference %(allFrameDxName, 
                                                      outputDir+'/'+thisClusterDxName, 
                                                      outputDir+'/'+thisDifferenceDxName, 
                                                      outputDir+'/'+thisDifferenceDxName,
                                                      outputDir+'/'+thisCentroidPdbName)
            plotAllContrib = plotAllContrib.replace('!MOLID!',str(clusterIndex))

            plotAll += plotAllContrib
        
        with open('%svisualizeAll.vmd' %(outputPrefix),'wb') as of:
            of.write(plotAll)

        ## Write gobstopper view script
        gobstopperViewHeader = '''

display projection Orthographic
color Display Background white
material add copy RTChrome
material change ambient Material23 0.00000
material change diffuse Material23 1.00000
material change specular Material23 0.00000
material change shininess Material23 0.00000
material change mirror Material23 0.00000
material change opacity Material23 0.00000
material change outline Material23 0.00000
material change outlinewidth Material23 0.00000
material change transmode Material23 1.00000


'''
        templateGobstopperViewScript = '''

mol new {!!!CLUSTER AVERAGE FILENAME!!!} waitfor all
mol modstyle 0 top Isosurface 0.7500000 0 0 1 1 1
#mol modstyle 0 !MOLID! Isosurface 0.2500000 0 1 1 1 1
#white
mol modcolor 0 top ColorID 8

mol addrep top
mol modstyle 1 top Isosurface 0.250000 1 0 1 1 1
#blue
mol modcolor 1 top ColorID 0 
mol showrep top 1 0 


'''
            
    # cluster parameter is a list of lists - each list is a cluster
    # Return values:
    # spread 
    # cnum - a value indicating the number of clusters excluding clusters with only 1 frame
    def find_spreads(self,clusters):

#        print cluster
        spreads = []
        cnum = 0
        newWay = True
        #print 'AAAA'
        if newWay == True:
            for cluster in clusters:
                
                thisSpread = numpy.sum(self.one_minus_overlap_values[cluster,:][:,cluster])/2
                spreads.append(thisSpread)
                
        else:
            for current_set in clusters:
                curr_spread = 0
    #            print current_set
                # All combinations of frames in current cluster
                #print current_set
                combinations = itertools.combinations(current_set,2)
        
                #NEED TO USE FIRST AND SECOND VALUES OUT OF ITERATOR!!!!!!!! 
                for frame1, frame2 in combinations:
                    curr_spread += self.one_minus_overlap_values[frame1][frame2]
                
                # Calculate the N(N-1)/2 denominator for the spread of a cluster
                # SET SPREAD TO 1 IF THERE ARE NO ELEMENTS OR ONLY A SINGLE ELEMENT 
                # IN THE CLUSTER
                if len(current_set) <= 1:
                    spreads.append(0)
                else:
                    cnum += 1
                    curr_spread /= (len(current_set)*(len(current_set)-1)/2)
                    spreads.append(curr_spread)
                

        ## Unexpected numbers of clusters are now handled in the Kelley penalty code segment
        #return spread,cnum
        return spreads
    
'''        
    # DO ALL CALCULATIONS IN ONE METHOD OR DO SEPARATE METHODS??? 
    def average_spread(self,spread,cnum):
        avg_value = 0
        for i in spread:
            avg_value += i
        
        avg_value = avg_value / cnum
        return avg_value
    
    def norm_avg_spread(self, list_avg_spread):
        max_avg_spread = list_avg_spread[0]
        min_avg_spread = list_avg_spread[0]
        
        for i in list_avg_spread:
            if i > max_avg_spread:
                max_avg_spread = i
            elif i < min_avg_spread:
                min_avg_spread = i
        
        
        return
'''

#def print_help():
#    print "To run cluster.py: cluster.py [optional -h -k] binding_overlap_file pdb_trajectory_file output_file_names"

class main():

    def __init__(self,argv):
        '''    TEMP INPUT: clsuter.py overlap_file original_traj_file '''

        ''' Pick between running kmeans or hierarchical clustering or both
            Parse the command line inputs.
        '''

        parser = argparse.ArgumentParser(description="Cluster POVME pocket volumes.")
        parser.add_argument('-m', 
                            help='The pocket overlap matrix (generated by binding_site_overlap.py)')
        parser.add_argument('-t', nargs='?', action='append',default=[],
                            help='A mapping between .npy file prefixes and their original pdb file/trajectory, separated by a colon. Can be used repeatedly. Ex: -t 1BYQ:./trajectories/1BYQ.pdb -t 1UYF:./trajectories/1UYF.pdb')
        parser.add_argument('-T', nargs='?', action='append', default=[],
                            help='A file containing a series of -t arguments')
        parser.add_argument('-i', nargs='?', 
                            help='The index file mapping the pocket overlap matrix to frame numbers. Required to return cluster representatives.')
        parser.add_argument('--kmeans', action='store_true', 
                            help='Use kmeans clustering instead of hierarchical.')
        parser.add_argument('-n', nargs='?', 
                            help='Manually set number of clusters. Otherwise the Kelley penalty will calculate the optimal number.')
        parser.add_argument('-N', nargs='?', 
                            help='Set min, min:max, or min:max:skip values for number of clusters that the Kelley penalty can consider.')
        parser.add_argument('-o', nargs='?', default='',
                            help='Specify an output prefix.')
        parser.add_argument('-a', action='store_true',
                            help='Output all frames into cluster subdirectories (not just cluster reps).')
        
        args = parser.parse_args(sys.argv[1:])
        #''' Initial options '''
        #command_input = {}
        #command_input['kmeans'] = False
        #command_input['hierarchical'] = False
        #command_input['csv_file'] = ''
        #command_input['output_name'] = ''
        #command_input['num_clusters'] = None
        #command_input['indexToFrames'] = ''

        #'''Quick and dirty hack for options - need to find more efficient way '''

        #for arg in argv:
        #    print arg
        #    if 'indexMapToFrames.csv' in arg:
        #        command_input['indexToFrames'] = arg
        #    elif '.csv' in arg:
        #        command_input['csv_file'] = arg
        #    elif arg == "-k":
        #        command_input['kmeans']= True
        #    elif arg == "-h":
        #        command_input['hierarchical'] = True
        #    elif arg.isdigit():
        #        command_input['num_clusters'] = int(arg)

        # Print message and exit out of program if missing essential files
        if args.m == '':
            print "Cannot run cluster.py: Need an overlap file from binding_site_overlap.py in order to cluster \n"
            #print_help()
            sys.exit(1)

        
        #if command_input['indexToFrames'] =='':
        #print args.i
 

        
#        if command_input['pdb_file'] == '':
#            print "Cannot run cluster.py: Need the initial trajectory file in order to separate into clusters \n"
#            print_help()
#            sys.exit(1)
                
        ''' Currently only matches correctly if you run script from the folder where all the npy files are located '''
#        for filename in os.listdir('.'):
#            print filename
#            if fnmatch.fnmatch(filename,input_string_file):
#                command_input['traj_file'].append(filename)

        # If both -k and -h weren't specified, then we want to allow both options
        ##if command_input['kmeans'] == command_input['hierarchical']:
        ##    command_input['kmeans'] = True
        ##    command_input['hierarchical'] = True

#        command_input['output_name'] = command_input['pdb_file'].strip('.pdb')

        # Read csv overlap file and parse results
        csv_input = InputReader()
        #csv_input.read_traj(command_input['indexToFrames'])
        #csv_input.read_overlap(command_input['csv_file'])

        csv_input.read_overlap(args.m)
        #if args.i != None:
        csv_input.read_indexFile(args.i)
        csv_input.parse_traj_inputs(args.t, args.T)

        #print csv_input.prefixToTrajectory
        #1/0
        #else:
            #If the user didn't specify an index file, make a dictionary that just returns the input number
            #nFrames = len(csv_input.overlapMatrix)
            #for i in range(nFrames):
            #    csv_input.frameToFileName[i]=i

        #coordinates = Cluster(csv_input.coordinates,csv_input.frames,csv_input.overlapMatrix,csv_input.frameToFileName)
        clustering_obj = Cluster(csv_input)
        #print args.n
        if args.n != None:
            if args.N != None:
                raise Exception('Both -n and -N command line options specified.')
            #If the user manually specified a number of clusters
            if args.kmeans == True:
                list_of_clusters = clustering_obj.kmeans_cluster(int(args.n))
            else:
                list_of_clusters = clustering_obj.hierarchical_cluster(int(args.n))
            #clusters = clustering_obj.separate_clusters(frame_assignments)
        # If the user didn't specify the number of clusters...
        else:
            # ...use the kelley penalty to find the optimal number
            if args.N != None:
                argsNsp = args.N.split(':')
                if len(argsNsp) == 1:
                    maxKPClusters = int(argsNsp[0])
                    userNClusters = range(1,maxKPClusters+1)
                    print "Computing Kelley penalty for nClusters from 1 to %i" %(maxKPClusters)
                elif args.N.count(':') == 1:
                    minKPClusters = int(argsNsp[0])
                    maxKPClusters = int(argsNsp[1])
                    userNClusters = range(minKPClusters,maxKPClusters+1)
                    print "Computing Kelley penalty for nClusters from %i to %i" %(minKPClusters, maxKPClusters)
                elif args.N.count(':') == 2:
                    minKPClusters = int(argsNsp[0])
                    maxKPClusters = int(argsNsp[1])
                    stride = int(argsNsp[2])
                    userNClusters = range(minKPClusters,maxKPClusters+1,stride)
                    print "Computing Kelley penalty for nClusters from %i to %i, taking strides of %i" %(minKPClusters, maxKPClusters, stride)
                    
            else:
                #maxKPClusters = clustering_obj.frames
                maxKPClusters = min(75, clustering_obj.frames)
                userNClusters = range(1,maxKPClusters+1)
                print "Computing Kelley penalty for nClusters from 1 to %i" %(maxKPClusters)
                
                
            ## In order to achieve proper scaling, we must ALWAYS have 1 as a possible cluster number 
            ## in the Kelley penalty computations
            if not 1 in userNClusters:
                potentialNClusters = [1] + userNClusters
            else:
                potentialNClusters = userNClusters
            
            ## We'll be doing numpy-style slicing later so convert it here
            potentialNClusters = numpy.array(potentialNClusters)
                                             
            clustering_results = {}
            #avSpreads = numpy.zeros(maxKPClusters+1)
            avSpreads = []
            # Invalidate "0 clusters" option
            #avSpreads[0] = -1
            
            
            lastProgressDecile = 0
            for index, nClusters in enumerate(potentialNClusters):
                progressDecile = (10*index)/len(potentialNClusters)
                if progressDecile > lastProgressDecile:
                    lastProgressDecile = progressDecile
                    print "Kelley penalty " + str(progressDecile*10) + "% computed"
                if args.kmeans == True:
                    list_of_clusters = clustering_obj.kmeans_cluster(nClusters)
                else:
                    list_of_clusters = clustering_obj.hierarchical_cluster(nClusters)
                
                    
                clustering_results[nClusters] = list_of_clusters
                #clusters = clustering_obj.separate_clusters(frame_assignments)
                if len(list_of_clusters) != nClusters:
                    # If we didn't get as many clusters as we expected, put a placeholder in the array
                    #avSpreads[nClusters] = -1
                    avSpreads.append(-1)
                    # and then skip to the next iteration
                    continue
                
                spreads = clustering_obj.find_spreads(list_of_clusters)
                nSingletonClusters = sum([len(i) == 1 for i in list_of_clusters])
                avSpread = float(sum(spreads)) / (nClusters - nSingletonClusters)
                #avSpreads[nClusters] = avSpread
                avSpreads.append(avSpread)
                
            avSpreads = numpy.array(avSpreads)
            
            ## Remove places where the spread is -1
            # Make boolean index array validSpreads (eg; [0 1 1 0 1] )
            validSpreads = avSpreads != -1
            # find indices of valid spreads (eg; [1 2 4] for above)
            validIndices = numpy.nonzero(validSpreads)[0]

            ## Remove invalid nClusters and avSpread values
            validNClusters = potentialNClusters[validIndices]
            avSpreads = avSpreads[validSpreads]
            ## Now normalize spreads to the range (1, N-1)
            # subtract to bring the minimum value to 0
            avSpreads -= numpy.min(avSpreads)
            # multiply to bring the max value to N-2
            avSpreads *= (clustering_obj.frames-2)/numpy.max(avSpreads)
            # Then add 1 to everything to shift the range to (1, N-1)
            avSpreads += 1
            
            ## and finally compute the penalty value
            kPenalties = avSpreads + (1.0*validNClusters)
            
            #pylab.scatter(validNClusters, kPenalties)
            if 1 in userNClusters:
                pylab.plot(validNClusters, kPenalties, '-o')
            else:
                pylab.plot(validNClusters[1:], kPenalties[1:], '-o')
            pylab.show()
            
            optimal_nClusters = validNClusters[numpy.argsort(kPenalties)[0]]
            list_of_clusters = clustering_results[optimal_nClusters]
            #clusters = clustering_obj.separate_clusters(frame_assignments)
            print "Done computing Kelley penalty. Optimal number of clusters is %i" %(optimal_nClusters)

        clusterInd2CentFileName = clustering_obj.find_centroids(list_of_clusters, args.o)

        ## If desired, output all frames in this cluster
        if args.a == True:
            clustering_obj.outputAllFrames(list_of_clusters, args.o)
        
        ## Generate cluster characteristics
        if (clustering_obj.indexToFrame != {}) and (clustering_obj.prefixToTrajectory != {}):
            clustering_obj.generate_difference_maps(list_of_clusters, clusterInd2CentFileName, args.o)
        


if __name__ == "__main__": main(sys.argv)




















